#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : lora
# @Time         : 2025/7/15 12:03
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.io.files_utils import to_bytes, to_url
from meutils.apis.utils import make_request_httpx
from meutils.llm.openai_utils import create_chat_completion, create_chat_completion_chunk
from meutils.llm.openai_utils.billing_utils import get_billing_n, billing_for_async_task, billing_for_tokens
from meutils.schemas.image_types import ImageRequest, ImageEditRequest

base_url = "https://ai.gitee.com/v1"


# /images/edits
async def edit_gitee_ai_image(token: Optional[str] = None):
    token = token or "5PJFN89RSDN8CCR7CRGMKAOWTPTZO6PN4XVZV2FQ"  # 替换为您的 Bearer Token

    # --- 请在这里配置您的参数 ---
    image_path = "/path/to/image.png"  # 替换为您的图片实际路径
    # --------------------------

    headers = {
        "Authorization": f"Bearer {token}",
        # httpx 会自动为 multipart/form-data 设置 Content-Type，无需手动指定
    }

    data = {
        'prompt': "A sunlit indoor lounge area with a pool containing a flamingo",
        'model': "FLUX.1-Kontext-dev",
        'size': "1024x1024",
        'steps': "20",
        'guidance_scale': "2.5",
        'return_image_quality': "80",
        'return_image_format': "PNG",
        'lora_weights': json.dumps(lora_weights_data),  # 将字典转换为 JSON 字符串
        'lora_scale': "1",
    }

    # 构建 multipart 数据
    files = {
        'image': ('image.png', image_content, 'image/png'),
    }

    data = await make_request_httpx(
        base_url=upstream_base_url,
        path=f"/v1/{path}",
        payload=payload,
        params=params,
        data=data,
        files=files,
        headers=headers,

        debug=True

    )


# 运行异步函数
if __name__ == "__main__":
    asyncio.run(edit_gitee_ai_image())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/6/11 15:13
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import asyncio
import os

from meutils.pipe import *
from meutils.io.files_utils import to_url, to_base64
from meutils.decorators.retry import retrying
from meutils.apis.oneapi.utils import polling_keys

from meutils.llm.clients import AsyncClient
from meutils.schemas.openai_types import CompletionRequest
from meutils.schemas.video_types import VideoRequest, SoraVideoRequest
from meutils.schemas.image_types import ImageRequest

from meutils.db.redis_db import redis_aclient
from meutils.llm.check_utils import check_token_for_volc
# from meutils.llm.check_utils import check_token_for_volc_with_cache as check_token_for_volc

from meutils.config_utils.lark_utils import get_next_token_for_polling, get_series

from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status, BackgroundTasks

# Please activate
FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Z59Js10DbhT8wdt72LachSDlnlf?sheet=rcoDg7"


async def get_valid_token(tokens: Optional[list] = None, batch_size: Optional[int] = None, seed: int = 0,
                          recheck: bool = False):
    """返回 tokens
    api_key = np.random.choice(api_key.split())

    """
    for i in range(3):  # 最多轮询3次，
        _ = await polling_keys('volc')
        if check_token_for_volc(_):
            tokens = tokens or _
            break

    if not tokens:
        logger.debug("获取轮询key失败，走兜底")

    api_key = tokens or os.getenv('VOLC_API_KEY') or await get_next_token_for_polling(
        feishu_url=FEISHU_URL,
        from_redis=True,
        ttl=24 * 3600,
        check_token=check_token_for_volc
    )
    if isinstance(api_key, list):  # 向下兼容
        api_key = '\n'.join(api_key)

    return api_key

    # tokens = tokens or await get_series(FEISHU_URL, duplicated=True)
    # batch_size = batch_size or 1
    #
    #
    # if seed == 0 and (volc_tokens := await redis_aclient.get(f"volc_tokens")):  # 刷新
    #     volc_tokens = volc_tokens.decode()
    #     if recheck:
    #         valid_tokens = await check_token_for_volc(volc_tokens.split('\n'))
    #
    #         volc_tokens = '\n'.join(valid_tokens)
    #
    #     return volc_tokens
    #
    # valid_tokens = []
    # for token in tokens:
    #     if token := await check_token_for_volc([token]):
    #         valid_tokens += token
    #
    #         logger.debug(valid_tokens)
    #         if len(valid_tokens) == batch_size:
    #             _ = '\n'.join(valid_tokens)
    #             await redis_aclient.set(f"volc_tokens", _, ex=2 * 3600)
    #             # ttl =await redis_aclient.ttl(f"volc_tokens")
    #
    #             return _


# check_image_and_video = partial(check, purpose='video and image')


@retrying()
async def create_task(request: Union[CompletionRequest, VideoRequest, SoraVideoRequest, ImageRequest],
                      api_key: Optional[str] = None):
    # api_key = api_key or await get_next_token_for_polling(feishu_url=FEISHU_URL, check_token=check)
    # api_key = api_key or await get_valid_token()
    #
    # api_key = np.random.choice(api_key.split('\n'))  # todo 轮询

    api_key = api_key or await polling_keys('volc')
    api_key = api_key or await get_next_token_for_polling(
        feishu_url=FEISHU_URL,
        from_redis=True,
        ttl=24 * 3600,
        check_token=check_token_for_volc
    )
    logany(request)

    logger.debug(f"api_key: {api_key}")
    if isinstance(request, SoraVideoRequest):  # todo 首尾帧 参考图
        # logger.debug(request.input_reference)
        # logger.debug(request.first_frame_image)
        # request.input_reference # 数组

        if "lite" in request.model and (
                request.input_reference or request.first_frame_image or request.last_frame_image
        ):
            request.model = request.model.replace("t2v", "i2v")

        request.prompt = request.prompt.split('--', maxsplit=1)[0]  # 取消命令行参数
        request.prompt = f"{request.prompt} --dur {request.seconds}"
        if request.ratio:
            request.prompt += f" --rt {request.ratio}"
        if request.resolution:
            request.prompt += f" --rs {request.resolution}"

        payload = {
            "model": request.model,
            "return_last_frame": True,
            # "callback_url": request.callback_url,
            "callback_url": request.callback_url or f"""{os.getenv("WEBHOOK_URL")}/seedance""",

            "service_tier": "default",

            "content": [
                {
                    "type": "text",
                    "text": request.prompt
                }
            ]
        }
        if request.first_frame_image:
            payload["content"] += [
                {
                    "role": "first_frame",
                    "type": "image_url",
                    "image_url": {
                        "url": request.first_frame_image
                    },
                }
            ]
        if request.last_frame_image:
            payload["content"] += [
                {
                    "role": "last_frame",
                    "type": "image_url",
                    "image_url": {
                        "url": request.last_frame_image
                    },
                }
            ]

        if request.input_reference:  # url
            payload["content"] += [
                {
                    "role": "reference_image",
                    "type": "image_url",
                    "image_url": {
                        "url": image
                    },
                }

                for image in request.input_reference if image
            ]



    elif isinstance(request, VideoRequest):  # 兼容jimeng
        request.prompt = f"{request.prompt.replace('--', '')} --rt {request.aspect_ratio} --dur {request.duration} --rs 1080p"

        payload = {
            "model": "doubao-seedance-1-0-lite-t2v-250428",
            "service_tier": "default",

            "content": [
                {
                    "type": "text",
                    "text": request.prompt
                }
            ]
        }
        if request.image_url:
            payload = {
                "model": "doubao-seedance-1-0-lite-i2v-250428",

                "content": [
                    {
                        "type": "text",
                        "text": request.prompt
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.image_url
                        }
                    }
                ]
            }
        if request.image_url and request.tail_image_url:
            payload = {
                "model": "doubao-seedance-1-0-lite-i2v-250428",
                "service_tier": "default",

                "content": [
                    {
                        "type": "text",
                        "text": request.prompt
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.image_url
                        },
                        "role": "first_frame"
                    },
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": request.tail_image_url
                        },
                        "role": "last_frame"
                    }
                ]
            }

        if "-pro" in request.model:
            payload['model'] = "doubao-seedance-1-0-pro-250528"  # 未来注销
    elif isinstance(request, ImageRequest):
        text = request.prompt if request.prompt.startswith("--") else ""

        payload = {
            "model": request.model,
            "service_tier": "default",

            "content": [
                {
                    "type": "text",
                    "text": text or "--subdivisionlevel medium --fileformat glb"
                },
                {
                    "type": "image_url",
                    "image_url": {
                        "url": request.image_urls[-1]
                    }
                }
            ]
        }

    else:

        payload = {
            "model": request.model,
            "service_tier": "default",

        }

        if hasattr(request, 'content'):
            payload["content"] = request.content

        elif image_urls := request.last_urls.get("image_url"):
            if payload["model"] not in {"doubao-seedance-1-0-lite-i2v-250428", "doubao-seedance-1-0-pro-250528"}:
                payload["model"] = "doubao-seedance-1-0-lite-i2v-250428"

            payload["content"] = [
                {
                    "type": "text",
                    "text": request.last_user_content
                },
                {
                    "type": "image_url",
                    "image_url": {
                        "url": image_urls[-1]
                        # "url": await to_url(image_urls[-1], filename=".png")

                    }
                }]
        else:
            payload["content"] = [
                {
                    "type": "text",
                    "text": request.last_user_content
                }]

    logany(bjson(payload))

    client = AsyncClient(base_url="https://ark.cn-beijing.volces.com/api/v3", api_key=api_key)

    response = await client.post(
        path="/contents/generations/tasks",
        cast_to=object,
        body=payload
    )

    if task_id := response.get('id'):
        await redis_aclient.set(task_id, api_key, ex=7 * 24 * 3600)

    return response  # {'id': 'cgt-20250611152553-r46ql'}


async def get_task(task_id: str, api_key: Optional[str] = None):
    if isinstance(task_id, list):
        tasks = [get_task(_, api_key) for _ in task_id]
        return asyncio.gather(*tasks)

    if api_key is None:
        token = api_key or await redis_aclient.get(task_id)  # 绑定对应的 token
        api_key = token and token.decode()
        if not token:
            raise HTTPException(status_code=404, detail="TaskID not found")

    client = AsyncClient(base_url="https://ark.cn-beijing.volces.com/api/v3", api_key=api_key)

    response = await client.get(
        path=f"/contents/generations/tasks/{task_id}",
        cast_to=object,
    )

    return response


async def get_task_from_feishu(task_id: Union[str, list], tokens: Optional[list] = None):  # todo: 定时校验
    feishu_url = "https://xchatllm.feishu.cn/sheets/Z59Js10DbhT8wdt72LachSDlnlf?sheet=rcoDg7"
    tokens = tokens or await get_series(feishu_url)

    if isinstance(task_id, list):
        tasks = [get_task_from_feishu(_, tokens) for _ in task_id]
        return await asyncio.gather(*tasks)

    if not await redis_aclient.get(task_id):

        for api_key in tqdm(tokens):
            client = AsyncClient(base_url="https://ark.cn-beijing.volces.com/api/v3", api_key=api_key)
            try:
                response = await client.get(
                    path=f"/contents/generations/tasks/{task_id}",
                    cast_to=object,
                )
                logger.debug(f"{task_id} => {api_key}")

                await redis_aclient.set(task_id, api_key, ex=7 * 24 * 3600)
                break

            except Exception as e:
                # logger.error(e)
                continue


# 执行异步函数
if __name__ == "__main__":
    # api_key = "07139a08-44e2-ba31-07f379bf99ed"  # {'id': 'cgt-20250611164343-w2bzq'} todo 过期调用get

    api_key = "c2449725-f758-42af-8f2c-e05b68dd06ad"  # 欠费

    # api_key = None

    request = CompletionRequest(
        model="doubao-seedance-1-0-pro-250528",
        # model="doubao-seaweed-241128",
        messages=[
            {"role": "user",
             "content": "无人机以极快速度穿越复杂障碍或自然奇观，带来沉浸式飞行体验  --resolution 1080p  --duration 5 --camerafixed false"}
        ],
    )
    request = VideoRequest(
        model="doubao-seedance-1-0-pro-250528",
        prompt="无人机以极快速度穿越复杂障碍或自然奇观，带来沉浸式飞行体验",
        duration=10
    )
    # r = arun(create_task(request, api_key))
    # r = {'id': 'cgt-20250612172542-6nbt2'}

    # arun(get_task(r.get('id')))

    # arun(get_task("cgt-20250707162431-smhwc"))

    # arun(get_task("cgt-20250707160713-j8kll"))

    tokens = arun(polling_keys('', channel_id=21385))

    tokens = list(set(tokens)) + ['68b877dc-a337-4a20-9091-738bb0fcd79c']
    from meutils.apis.oneapi.tasks import get_tasks

    ids = arun(get_tasks(return_ids=True))

    ids = ['cgt-20260126144127-hghhr']


    arun(get_task_from_feishu(ids, tokens))

    # arun(get_valid_token())

    # request = ImageRequest(
    #     model="doubao-seed3d-1-0-250928",
    #     prompt="无人机以极快速度穿越复杂障碍或自然奇观，带来沉浸式飞行体验",
    #     image=[
    #         "https://ark-project.tos-cn-beijing.volces.com/doc_image/seed3d_imageTo3d.png"
    #     ]
    # )
    #
    # r = arun(create_task(request, None))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kuaishou_types
# @Time         : 2024/7/9 13:26
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# oss
"https://s2-111386.kwimgs.com/bs2/mmu-kolors-public/5f9c7b42688fe95c4b8d9ebd1dba3431.png?x-oss-process=image/resize,m_mfit,w_305"

# BASE_URL = "https://klingai.kuaishou.com"
# UPLOAD_BASE_URL = "https://upload.kuaishouzt.com"
# DOWNLOAD_URL= "https://klingai.kuaishou.com/api/works/batch_download_v2"

BASE_URL = "https://klingai.com"
UPLOAD_BASE_URL = "https://upload.uvfuns.com"
DOWNLOAD_URL = "https://klingai.com/api/works/batch_download_v2"  # /api/works/batch_download_v2?workIds=100331495&fwm=false

FEISHU_URL = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=XuxPYK"
FEISHU_URL_VIP = "https://xchatllm.feishu.cn/sheets/GYCHsvI4qhnDPNtI4VPcdw2knEd?sheet=hw8MXS"

ASPECT_RATIOS = {
    "1:1": "1024x1024",

    "16:9": "1344x768",
    "9:16": "768x1344",

    "4:3": "1152x896",
    "3:4": "896x1152",

    "3:2": "1216x832",
    "2:3": "832x1216",

}

STYLES = {
    "默认", "皮克斯", "卡通盲盒", "新海诚", "动漫3D", "怀旧动漫", "电子游戏", "水彩插画", "莫奈油画", "高清写实"
}

STATES = {
    5: "queue",
    10: "processing",
    99: "completed",
    50: "failed",
}


class KolorsRequest(BaseModel):
    prompt: str = '多肉植物，带着水珠，潮玩盲盒风格，皮克斯，3D质感，温馨的环境，丰富的场景，最佳画质，超精细，Octane渲染'
    negativePrompt: str = ''  # 不希望出现的内容

    style: str = "默认"
    imageCount: int = 1
    resolution: str = "1024x1024"

    #
    aspect_ratio: Literal["1:1", "2:3", "3:2", "3:4", "4:3", "9:16", "16:9"] = "1:1"

    # 垫图
    referImage: str = ''
    fidelity: float = 0.5  # 参考度

    cfgScale: float = 7
    strength: float = 0.8

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        if self.style not in STYLES:
            self.style = "默认"

        if self.resolution not in set(ASPECT_RATIOS.values()):
            self.resolution = ASPECT_RATIOS.get(self.aspect_ratio, "1024x1024")

        if self.referImage and self.referImage.startswith("http"):
            self.imageCount = 4
            self.referImage = self.referImage.split("/")[-2].split('?')[0]


class KlingaiImageRequest(BaseModel):
    mode: str = "pro"

    prompt: str = '清凉夏季美少女，微卷短发，运动服，林间石板路，斑驳光影，超级真实，16K'
    style: str = "默认"
    aspect_ratio: Optional[Literal["1:1", "2:3", "3:2", "3:4", "4:3", "9:16", "16:9"]] = "1:1"

    n: int = 1

    url: Optional[str] = None

    image_fidelity: Optional[float] = None

    #
    payload: dict = {}

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        inputs = []
        task_type = "mmu_txt2img_aiweb"
        if self.url:
            task_type = "mmu_img2img_aiweb"
            inputs.append({'name': 'input', 'inputType': 'URL', 'url': self.url})
            # inputs[-1]["fromWorkId"] = self.url.split('/')[-3].split('_')[-1]

        arguments = [
            {'name': 'prompt', 'value': self.prompt},
            {'name': 'style', 'value': self.style},
            {'name': 'aspect_ratio', 'value': self.aspect_ratio or "1:1"},
            {'name': 'imageCount', 'value': self.n},
            {"name": "fidelity", "value": self.image_fidelity or 0.5},
            {'name': 'biz', 'value': 'klingai'},
            {"name": "kolors_version", "value": "1.5"} ####### 1.5
        ]
        self.payload = {
            'arguments': arguments,
            'type': task_type,
            'inputs': inputs
        }


class Camera(BaseModel):
    type: Literal[
        "empty", "horizontal", "vertical", "zoom", "tilt", "pan", "roll", "down_back",
        "simple", "down_back", "forward_up", "left_turn_forward", "right_turn_forward",
    ] = "empty"
    horizontal: Optional[int] = 0
    vertical: Optional[int] = 0
    zoom: Optional[int] = 0
    tilt: Optional[int] = 0
    pan: Optional[int] = 0
    roll: Optional[int] = 0
    # """水平运镜，可选，取值范围：[-10, 10]"""
    # horizontal: Optional[int] = None
    # """水平摇镜，可选，取值范围：[-10, 10]"""
    # pan: Optional[int] = None
    # """旋转运镜，可选，取值范围：[-10, 10]"""
    # roll: Optional[int] = None
    # """垂直摇镜，可选，取值范围：[-10, 10]"""
    # tilt: Optional[int] = None
    # """垂直运镜，可选，取值范围：[-10, 10]"""
    # vertical: Optional[int] = None
    # """变焦，可选，取值范围：[-10, 10]"""
    # zoom: Optional[int] = None

    camera_json: str = ""  # 运镜

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        if self.type == "down_back":
            self.vertical = self.vertical or -10
            self.zoom = self.zoom or -10

        if self.type == "SIMPLE":
            self.type = "empty"
        # elif hasattr(self, self.type):
        #     self.__setattr__(self.type, self.__getattr__(self.type))

        self.camera_json = json.dumps(self.dict(exclude={'camera_json'}))


class KlingaiVideoRequest(BaseModel):
    """增加计费模式，支持高质量模式
    kling-v1.0-std-5s

    kling-v1.5-std-5s 20积分
    kling-v1.5-std-10s

    kling-v1.5-pro-5s 35积分
    kling-v1.5-pro-10s

    kling-v1.6-std-5s 20积分
    kling-v1.6-std-10s

    kling-v1.6-pro-5s 35积分
    kling-v1.6-pro-10s

    """
    model: str = 'kling-v1.6'
    mode: Literal["mini", "std", "pro"] = "std"

    prompt: str = ''
    negative_prompt: Optional[str] = ''  # 不希望出现的内容

    n: Literal[1, 2, 3, 4] = 1

    duration: Optional[int] = 5
    aspect_ratio: Literal["1:1", "2:3", "3:2", "3:4", "4:3", "9:16", "16:9"] = "16:9"
    cfg_scale: Optional[float] = 0.5  # 创意程度
    camera: Camera = Camera()  # 运镜

    url: Optional[str] = ''
    # 图生视频
    tail_image_url: Optional[str] = ''

    #
    payload: dict = {}

    def __init__(self, /, **data: Any):
        super().__init__(**data)

        if self.duration > 5:
            self.duration = 10
        else:
            self.duration = 5

        if self.tail_image_url or self.mode == 'mini':
            self.duration = 5  # 尾帧仅支持 5s

        arguments = [
            {'name': 'prompt', 'value': self.prompt},
            {'name': 'negative_prompt', 'value': self.negative_prompt},
            {'name': 'duration', 'value': self.duration},
            {'name': 'aspect_ratio', 'value': self.aspect_ratio},
            {"name": "imageCount", "value": self.n},
            {'name': 'cfg', 'value': self.cfg_scale},
            {'name': 'camera_json', 'value': self.camera.camera_json},

            {'name': 'biz', 'value': 'klingai'},
        ]

        kling_version = '1.5'
        if self.model == 'kling-v1.6':
            kling_version = "1.6"
            arguments.append({"name": "camera_control_enabled", "value": "false"})  # 暂不支持
        else:
            arguments.append({"name": "camera_control_enabled", "value": "true"})

        arguments.append({"name": "kling_version", "value": kling_version})

        inputs = []
        task_type = "m2v_txt2video"
        if self.url:
            task_type = "m2v_img2video"
            inputs.append({'name': 'input', 'inputType': 'URL', 'url': self.url})
            if self.url.endswith(".mp4"):  # 待测试
                inputs[-1]["fromWorkId"] = self.url.split('/')[-3].split('_')[-1]
                task_type = "m2v_extend_video"

        if self.tail_image_url:
            task_type = "m2v_img2video"  # 尾帧 高质量模式

            arguments.append({"name": "tail_image_enabled", "value": "true"})
            inputs.append({'name': 'tail_image', 'inputType': 'URL', 'url': self.tail_image_url})

        self.payload = {
            'arguments': arguments,
            'type': f"""{task_type}{'_hq' if self.mode == 'pro' else ''}""",
            'inputs': inputs
        }

    class Config:
        json_schema_extra = {
            "examples": [
                {
                    "prompt": "一只可爱的黑白边境牧羊犬，头伸出车窗，毛发被风吹动，微笑着伸出舌头",

                    "url": "",
                    "tail_image_url": "",

                    "negative_prompt": "",
                    "duration": 5,
                    "aspect_ratio": "16:9",
                    "cfg": 0.5,
                    "camera": {
                        "type": "empty",
                        "horizontal": 0,
                        "vertical": 0,
                        "zoom": 0,
                        "tilt": 0,
                        "pan": 0,
                        "roll": 0
                    }
                },

                {
                    "prompt": "一只可爱的黑白边境牧羊犬，头伸出车窗，毛发被风吹动，微笑着伸出舌头",

                    "url": "https://p2.a.kwimgs.com/bs2/upload-ylab-stunt/ai_portal/1720681052/LZcEugmjm4/whqrbrlhpjcfofjfywqqp9.png",
                    "tail_image_url": "",

                    "negative_prompt": "",
                    "duration": 5,
                    "aspect_ratio": "16:9",
                    "cfg": 0.5,
                    "camera": {
                        "type": "empty",
                        "horizontal": 0,
                        "vertical": 0,
                        "zoom": 0,
                        "tilt": 0,
                        "pan": 0,
                        "roll": 0
                    }
                },

                {
                    "prompt": "开花过程",

                    "url": "https://h2.inkwai.com/bs2/upload-ylab-stunt/ai_portal/1721561429/3z7bHmSC1Y/3yrmpqh7typspfcchmepar.png",
                    "tail_image_url": "https://p2.a.kwimgs.com/bs2/upload-ylab-stunt/ai_portal/1721561524/GPoJWxBS8s/y9wlbuku3exeo7ra85s4su.png",

                    "negative_prompt": "",
                    "duration": 5,
                    "aspect_ratio": "16:9",
                    "cfg": 0.5,
                    "camera": {
                        "type": "empty",
                        "horizontal": 0,
                        "vertical": 0,
                        "zoom": 0,
                        "tilt": 0,
                        "pan": 0,
                        "roll": 0
                    }
                }

            ]
        }


"https://p2.a.kwimgs.com/bs2/upload-ylab-stunt/special-effect/output/HB1_PROD_ai_web_30121850/8784600431869866912/out.mp4"
if __name__ == '__main__':
    # print(Camera().camera_json)
    print(KlingaiVideoRequest.Config.json_schema_extra.get('examples')[-1])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : scheduler
# @Time         : 2024/1/9 08:51
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

from fastapi import status
from fastapi import APIRouter, File, UploadFile, Query, Form, Response, Request
from apscheduler.schedulers.asyncio import AsyncIOScheduler as Scheduler

router = APIRouter()

scheduler = Scheduler(timezone="Asia/Shanghai")


@router.on_event("startup")
async def start_scheduler():
    """scheduler.add_job(do_job, 'interval', seconds=3)
    - 结合playwright定时刷新cookies
    """
    # scheduler.add_job(do_job, **trigger_args)  # 注入任务

    scheduler.start()

# @router.on_event("startup")
# async def start_scheduler():
#     from meutils.decorators.schedulers import scheduled_job
#
#     @scheduled_job(trigger_kwargs={'seconds': 5})
#     def tick():
#         print(f"########TASK: {time.ctime()}")
#
#     tick()


# @router.on_event("startup")
# async def start_scheduler():
#     def tick():
#         print(time.ctime())
#     from apscheduler.schedulers.asyncio import AsyncIOScheduler as Scheduler
#     from apscheduler.schedulers.background import BackgroundScheduler as Scheduler
#
#     scheduler = Scheduler(timezone="Asia/Shanghai")
#     scheduler.add_job(tick, 'interval', seconds=3)
#     scheduler.start()


# from meutils.pipe import *
# from meutils.serving.fastapi import App
# from meutils.serving.fastapi.routers import scheduler
#
# if __name__ == '__main__':
#     app = App()
#     scheduler.scheduler.add_job(lambda: print("hello"), 'interval', seconds=3)
#
#     app.include_router(scheduler.router)
#     app.run(port=8899)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : _fastapi
# @Time         : 2023/4/28 13:08
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.str_utils import json_loads

from fastapi.responses import Response, StreamingResponse
from fastapi import FastAPI, Form, Depends, File, UploadFile, Body, Request, BackgroundTasks


class App(FastAPI):

    def __init__(self, title: str = "Meutils API", **kwargs):
        super().__init__(title=title, **kwargs)

    def run(self, app=None, host="0.0.0.0", port=8000, workers=1, access_log=True, reload=False, app_dir=None, **kwargs):
        """
        :param app:   app字符串可开启热更新 debug/reload
        """
        import uvicorn

        uvicorn.config.LOGGING_CONFIG['formatters']['access']['fmt'] = f"""
              🔥 %(asctime)s - %(levelprefix)s %(client_addr)s - "%(request_line)s" %(status_code)s
              """.strip()
        uvicorn.run(
            app or self,
            host=host, port=port, workers=workers, access_log=access_log, reload=reload, app_dir=app_dir, **kwargs
        )

    def register(self, handler_func, path=None, methods=None, prefix='', **api_route_kwargs):  # add_route注册服务
        """

        :param handler_func: 自定义 handler_func
        :param path:
        :param methods:
        :return:
        """
        path = path or f"{handler_func.__name__.replace('_', '-')}"
        path = (Path('/') / Path(prefix) / Path(path)).as_posix()

        self.handler_func = handler_func
        self.api_route(path=path, methods=methods, **api_route_kwargs)(self.handler)

    async def handler(self, request: Request):  # 处理一切入参
        """可重写"""
        input = request.query_params._dict
        body = await request.body()

        if body.startswith(b'{'):  # 主要分支 # json={}
            input.update(json_loads(body))  # json_loads

        query = input.get('query', '')

        if inspect.isgeneratorfunction(self.handler_func):
            return StreamingResponse(self.handler_func(query), media_type='text/event-stream')  # 流式接口

        return self.handler_func(query)


if __name__ == '__main__':

    def gen_data(query):
        for i in range(5):
            time.sleep(i)
            yield f"{query} {i}\n"


    app = App()
    app.register(gen_data, '/x')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : doc
# @Time         : 2023/3/27 12:02
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


def doc2text(p):
    p = Path(p)
    import docx
    doc = docx.Document(p)
    text = ''
    for part in doc.paragraphs:
        _ = part.text.strip()
        if _:
            text += '\n' + _

    return p.parent.name, p.name, text.strip()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/10/17 22:55
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import json
import os

from meutils.pipe import *
from meutils.db.redis_db import redis_aclient
from meutils.schemas.video_types import SoraVideoRequest, Video
from meutils.apis.volcengine_apis import videos as volc_videos
from meutils.apis.aiml import videos as aiml_videos
from meutils.apis.gitee import videos as gitee_videos
from meutils.apis.hailuoai import openai_videos as hailuoai_videos
from meutils.apis.aiping import videos as aiping_videos
from meutils.apis.replicate import videos as replicate_videos

from meutils.apis.runware import videos as runware_videos  # todo 兼容

"""
失败原因可以 存在get任务里 方便退费
"""


class OpenAIVideos(object):

    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None):
        self.api_key = api_key
        self.base_url = base_url or ""
        logger.debug(f"base_url: {self.base_url}")  # 来源

    @cached_property
    def biz(self):
        parts = self.base_url.split(".")
        return (parts[1] if len(parts) >= 2 else self.base_url)[::-1]

    async def create(self, request: SoraVideoRequest):
        response = {}

        if "aiping" in self.base_url:
            response = await aiping_videos.Tasks(api_key=self.api_key).create(request)

        elif any(i in self.base_url for i in {"hailuo"}):
            response = await hailuoai_videos.Tasks(api_key=self.api_key).create(request)

        elif any(i in self.base_url for i in {"gitee", "moark"}):
            response = await gitee_videos.Tasks(api_key=self.api_key).create(request)

        elif "volc" in self.base_url:
            response = await volc_videos.create_task(request, self.api_key)  # {'id': 'cgt-20250611152553-r46ql'}

        elif "aimlapi" in self.base_url:
            response = await aiml_videos.Tasks(api_key=self.api_key, base_url=self.base_url).create(request)

        elif "replicate" in self.base_url:
            response = await replicate_videos.Tasks(api_key=self.api_key).create(request)

        if response and (task_id := (
                hasattr(response, "id") and response.id  # video
                or response.get("id")
                or response.get("task_id")
                or response.get("generation_id")
        )):
            task_id = task_id.replace("/", "@")
            # task_id = f"{self.biz}::{task_id}"  # 组装biz # todo base url  # 区分不同平台
            if not isinstance(response, Video):
                response = Video(id=task_id)  # 只要有id 就会进入队列 系统bug

            if response.status == "failed":  # 提交任务失败：保存失败结果
                _response = copy.deepcopy(response)
                if isinstance(response.error, dict):

                    message = f"""{response.error.get("message", "")}\n\n"""
                    if "图片格式有问题，请上传新图片" in message:
                        message += "图片上传失败，长宽比需在 5:2 和 2:5 之间\n\n"
                    if "内容违反了Sora指南" in message:
                        message += """
                            内容违反了Sora指南:
                            1. 仅适合18岁以下观众的内容。
                            2. 受版权保护的角色和音乐无法生成。
                            3. 无法生成真实人物，包括公众人物。
                            4. 带有真人面部的图像无法生成。\n\n"""

                    if (_ := request.model_dump_json(indent=4, exclude_none=True)) and len(_) < 3000:
                        message += _
                    response.error["message"] = message

                #     logger.debug(response)
                # logger.debug(_response)
                await redis_aclient.set(f"request-failed:{task_id}", response.model_dump_json(), ex=3 * 24 * 3600)
                return _response

            if self.api_key:
                await redis_aclient.set(task_id, self.api_key, ex=7 * 24 * 3600)

            return response

    async def get(self, task_id):
        if _ := await redis_aclient.get(f"request-failed:{task_id}"):
            return json.loads(_)

        video = Video(id=task_id)
        if api_key := await redis_aclient.get(task_id):
            api_key = api_key.decode()
        else:
            raise ValueError(f"task_id not found: {task_id}")

        logger.debug(api_key)

        task_id = task_id.replace("@", "/")  # 还原

        if api_key.startswith("QC-"):
            video = await aiping_videos.Tasks(api_key=api_key, base_url=self.base_url).get(task_id)
            return video

        elif task_id.isnumeric and len(task_id) == 18 and str(task_id).startswith('4'):
            video = await hailuoai_videos.Tasks(api_key=api_key, base_url=self.base_url).get(task_id)
            return video

        elif api_key.startswith("r8_"):
            video = await replicate_videos.Tasks(api_key=api_key, base_url=self.base_url).get(task_id)
            return video

        elif len(api_key) == 40 and api_key.isupper():
            video = await gitee_videos.Tasks(api_key=api_key, base_url=self.base_url).get(task_id)
            return video

        elif task_id.startswith("cgt-"):
            if response := await volc_videos.get_task(task_id, api_key):
                # logger.debug(bjson(response))
                """
                {
                    "id": "cgt-20251225095015-5xmc2",
                    "model": "doubao-seedance-1-0-pro-250528",
                    "status": "failed",
                    "error": {
                        "code": "OutputVideoSensitiveContentDetected",
                        "message": "The request failed because the output video may contain sensitive information. Request id: 02176662741563700000000000000000000ffffac19188dffe699"
                    },
                    "created_at": 1766627415,
                    "updated_at": 1766627466,
                    "service_tier": "default",
                    "execution_expires_after": 172800
                }
                """

                video = Video(id=task_id, status=response, error=response.get("error"), metadata=response, progress=11)
                if video.status == "completed":
                    video.progress = 100
                    video.video_url = response.get("content", {}).get("video_url")  # 多个是否兼容

        elif len(api_key) == 32 and (":" in task_id and "/" in task_id or len(task_id) == 21):  # 粗判断
            video = await aiml_videos.Tasks(api_key=api_key, base_url=self.base_url).get(task_id)
            return video

        elif len(api_key) == 32 and len(task_id) == 36:  # 粗判断
            video = await runware_videos.get_task(task_id)
            return video

        return video


if __name__ == '__main__':
    api_key = "267a3b8a-ef06-4d8f-bd24-150f99bb17c1"
    model = "doubao-seedance-1-0-pro-fast-251015"

    api_key = "3993b09fccb542c5a46094bac9a4cf96"
    model = "openai/sora-2-t2v"
    model = "alibaba/wan2.5-i2v-preview"

    api_key = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NzIzNTA5NjUsInVzZXIiOnsiaWQiOiI0Njk4ODIxOTY3NDM1Mjg0NDkiLCJuYW1lIjoiYWZzbCBkcnF2IiwiYXZhdGFyIjoiIiwiZGV2aWNlSUQiOiIzMTE2NzAxODUwMDg0ODg0NTIiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.3pO0O36-um2fQs0ML0eHwpi0D7rV5yjmnjcpuiZcNKw"
    base_url = "hailuo"
    model = "nano-banana2_4k"
    # base_url = "sora"
    model = "sora-2-4s"

    request = SoraVideoRequest(
        model=model,
        prompt="裸体女孩",
        # model=f"{model}_480p",
        # model=f"{model}_720p",
        # model=f"{model}_1080p",

        # seconds="4",
        size="720x1280",
    )
    videos = OpenAIVideos(api_key=api_key, base_url=base_url)

    # video = arun(videos.create(request))

    # Video(id='cgt-20251031183121-zrt26', completed_at=None, created_at=1761906681, error=None, expires_at=None,
    #       model=None, object='video', progress=0, remixed_from_video_id=None, seconds=None, size=None, status='queued',
    #       video_url=None, metadata=None)

    task_id = ""
    task_id = "nltmquwNYRj6xNz-0PSaC"
    task_id = "SZiTAnMeOG7quoybO3VKk"

    task_id = "9888bd5a-b87c-40e3-8bbf-c8914464ecf0"
    task_id = "473537684238921736"
    arun(videos.get(task_id))

    # video = arun(videos.create(request))

    # task_id = "video_690dc20970808198b65cd9c04205edce0ed7e02d84c9579c:openai/sora-2-t2v"
    # task_id = "ee57044b-01e8-4aea-bd5f-48a03d653548:alibaba/wan2.5-t2v-preview"
    # task_id = "df489658-125b-4c65-a949-41d73c76cf0e:alibaba/wan2.5-t2v-preview"
    # arun(videos.get(task_id))

    # wJCWwQ5x0CcVIWGzXEp-A

    # {
    #     "id": "35f26664-b847-4821-b940-3f244a641bf7",
    #     "error": {
    #         "code": "4",
    #         "message": "error"
    #     },
    #     "model": "sora-2",
    #     "object": "video",
    #     "status": "failed",
    #     "created_at": 1640995200
    # }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : x
# @Time         : 2025/5/8 11:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *




def jwt_token_decode(jwt_token):
    # 提取 payload 部分
    payload_part = jwt_token.split('.')[1]

    # 对 payload 进行 Base64 解码
    try:
        # 尝试标准 Base64 解码
        decoded_payload = base64.b64decode(payload_part + '==', validate=True).decode('utf-8')
    except base64.binascii.Error:
        # 尝试 URL 安全的 Base64 解码
        decoded_payload = base64.urlsafe_b64decode(payload_part + '==').decode('utf-8')

    # 解析 JSON 字符串
    payload = json.loads(decoded_payload)

    return payload


if __name__ == '__main__':
    jwt_token="eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NDg3Mzg4MTQsInVzZXIiOnsiaWQiOiIyMjkwODQ3NTA2MDEzODgwMzciLCJuYW1lIjoi5bCP6J665bi9ODAzNyIsImF2YXRhciI6Imh0dHBzOi8vY2RuLmhhaWx1b2FpLmNvbS9wcm9kL3VzZXJfYXZhdGFyLzE3MDYyNjc3MTEyODI3NzA4NzItMTczMTk0NTcwNjY4OTY1ODk2b3ZlcnNpemUucG5nIiwiZGV2aWNlSUQiOiIyNDM3MTMyNTI1NDU5ODY1NjIiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.o0SoZMSTWkXNHxJjt3Ggby5MJWSfd-rnK_I95T_WMP8"
    print(jwt_token_decode(jwt_token))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : sk_utils
# @Time         : 2022/4/7 下午1:57
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : 

from sklearn.model_selection import train_test_split

# ME
from meutils.pipe import *


def train_valid_test_split(df, y=None, test_size=0.3, random_state=None):
    if y is None:
        df_train, df_test = train_test_split(df, test_size=test_size, random_state=random_state, stratify=y)
    else:
        df_train, df_test, _, y = train_test_split(df, y, test_size=test_size, random_state=random_state, stratify=y)

    df_val, df_test = train_test_split(df_test, test_size=0.5, random_state=random_state, stratify=y)
    return df_train, df_val, df_test




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : translator_types
# @Time         : 2024/7/18 14:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


class DeeplxRequest(BaseModel):
    text: str

    source_lang: str = "auto"
    target_lang: str = "ZH"

    class Config:
        frozen = True

        json_schema_extra = {
            "example": {
                "text": "火哥AI是最棒的",
                "source_lang": "auto",
                "target_lang": "EN"
            }
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : uniform_queue
# @Time         : 2023/12/6 17:59
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import time

from meutils.pipe import *
from queue import Queue, Empty


class UniformQueue(object):

    def __init__(self, generator: Generator):
        self.queue = Queue()

        self.producer(generator)

    def smooth(self, interval: float = 0.05, timeout: float = 30, break_fn: Callable = lambda item: item is None):
        return self.consumer(interval, timeout, break_fn)

    def consumer(self, interval: float = 0.066, timeout: float = 10, break_fn: Callable = lambda item: item is None):
        """

        :param interval:
        :param timeout:
        :param break_fn: 默认item为None跳出
            lambda line: line.choices[0].finish_reason == 'stop'
        :return:
        """
        while True:
            try:
                item = self.queue.get(timeout=timeout)
                yield item

                if break_fn(item):  # 跳出队列，很重要：最后一个会被返回【注意None值的处理】
                    break

            except Empty:
                break

            time.sleep(interval)

    @background
    def producer(self, generator):
        for i in generator:
            self.queue.put(i)


if __name__ == '__main__':

    def gen():
        while 1:
            yield '#'


    for i in tqdm(UniformQueue(gen()).consumer(interval=0.1)):
        print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/12/16 17:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from openai import AsyncClient
from meutils.pipe import *
from meutils.apis.jimeng.doubao_utils import generate_cookie, generate_params

from meutils.schemas.image_types import ImageRequest

from meutils.schemas.jimeng_types import BASE_URL, MODELS_MAP, FEISHU_URL
from meutils.config_utils.lark_utils import get_next_token_for_polling


async def create(token: Optional[str] = None):
    token = token or "712a47e7eec7c03b4cc7229775e06841"
    cookie = generate_cookie(token)
    params = generate_params()

    headers = {
        'Cookie': cookie,
        'agw-js-conv': 'str',
        'User-Agent': 'Apifox/1.0.0 (https://apifox.com)',
        'content-type': 'application/json'
    }
    payload = {
        "messages": [
            {
                "content": "{\"text\":\"一只猫\"}",
                "content_type": 2009,
                "attachments": [

                ]
            }
        ],
        "completion_option": {
            "is_regen": False,
            "with_suggest": False,
            "need_create_conversation": False,
            "launch_stage": 1,
            "is_replace": False,
            "is_delete": False,
            "message_from": 0,
            "event_id": "0"
        },
        "section_id": "6287920686327298",
        "conversation_id": "6287920686327042",
        "local_message_id": "936eee40-354d-11f0-83df-6b1810ffef8a"

        # "local_message_id": str(uuid.uuid4())

    }

    client = AsyncClient(base_url="https://www.doubao.com/samantha", default_headers=headers, api_key='xx',
                         )
    response = await client.post("/chat/completion", body=payload, cast_to=object, stream=True,
                                 options=dict(params=params))
    async for i in response:
        print(i)
    # return response


if __name__ == '__main__':
    arun(create())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : utils
# @Time         : 2026/1/20 10:36
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


import oss2

from meutils.pipe import *
from meutils.hash_utils import md5
from meutils.io.files_utils import to_bytes
from meutils.jwt_utils import decode_jwt_token
from meutils.schemas.hailuo_types import BASE_URL_ABROAD as BASE_URL

from meutils.schemas.hailuo_types import VideoRequest, VideoResponse
from meutils.llm.check_utils import check_tokens
from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message as _send_message, VIDEOS

from meutils.apis.hailuoai.yy import get_yy

APP_ID = '3001'
VERSION_CODE = '22203'

PARAMS = {
    'device_platform': 'web',
    'app_id': APP_ID,
    'version_code': VERSION_CODE,
    'biz_id': '0',
    'os_name': 'Mac',
    'browser_name': 'chrome',
    'device_memory': '8',
    'cpu_core_num': '10',
    'browser_language': 'zh-CN',
    'browser_platform': 'MacIntel',
    'screen_width': '1920',
    'screen_height': '1080',
    'lang': "zh-Intl",

    'uuid': '8c059369-00bf-4777-a426-d9c9b7984ee6',
    'device_id': '339114928865529864',
    'unix': f'{int(time.time())}000'
}


@alru_cache(ttl=1 * 24 * 60 * 60)
@retrying()
async def get_access_token(token: str):
    params = PARAMS.copy()

    payload = {}
    headers = {
        'Content-Type': 'application/json',
        'token': token,
        'yy': get_yy(payload, params=params, url="/v1/api/user/renewal")
    }

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post("/v1/api/user/renewal", params=params, content=json.dumps(payload))
        response.raise_for_status()
        logger.debug(response.json())
        return response.json()['data']['token']


@alru_cache(ttl=1 * 60)
async def get_request_policy(token):
    """
    {
        "data": {
            "accessKeyId": "STS.NUVNbdQfqTNixCxUTAhdeJToq",
            "accessKeySecret": "DRBsh8Qm8VnXXxTwFMX5KkXVoqbbsPj4ewEfgTLysGvM",
            "securityToken": "CAISiwN1q6Ft5B2yfSjIr5bjBdjQvLlQ44yCemXJsVQUZOtJpZHEkzz2IHhMf3VpAusWsPw1n2tT6/sdlrBoS4JMREPJN5EhtsQLrl75PdKY4Jzltedb0EIf6JFQUUyV5tTbRsmkZj+0GJ70GUem+wZ3xbzlD2vAO3WuLZyOj7N+c90TRXPWRDFaBdBQVH0AzcgBLinpKOqKOBzniXayaU1zoVhYiHhj0a2l3tbmvHi4tlDhzfIPrIncO4Wta9IWXK1ySNCoxud7BOCjmCdb8EpN77wkzv4GqzLGuNCYCkkU+wiMN+vft9ZjKkg7RNBjSvMa8aWlzKYn57OCyt2v8XsXY7EJCRa4bZu73c7JFNmuMtsEbrvhMxzPqIvXbsKp6lh/MSxDblgRIId8dWURExUpTSrBIaOh6M4Bo5NbzHzuOsgSpnkVpz2AlbLiT9M/1aieRiRTcymwO/ayjeq6CeAF3mM8Mm0qPRouTM2+Zo5YD3N1opjTpiapdUYLox8awbuQLp25tMiF6FiLDvouuRqAAQJZHVcOeb5qnR6mkzw5hwzSOXoMXVFzDE2aB7dvRYFD9HiG6T66hE4Xlfpph9H7xWrpaBf5vqHQXp4gyuqVOgFIjdPECwisXlyAKQWMak7bGToh3cetCux3pjq74sP/KAjzSzDkwciJBbn8vZzNKKR/ozxqve925vtPPPWGPowwIAA=",
            "expiration": "2024-10-22T02:55:14Z",
            "dir": "cdn-yingshi-ai-com/prod/2024-10-22-09/user/multi_chat_file",
            "endpoint": "oss-cn-wulanchabu.aliyuncs.com",
            "bucketName": "minimax-public-cdn",
            "serverTime": "2024-10-22T02:10:07Z"
        }
    }
    """
    # token = await get_access_token(token)

    headers = {
        'token': token,
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36',
    }


    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.get("/v1/api/files/request_policy")
        response.raise_for_status()
        return response.json()['data']


async def upload(file: bytes, token: str):
    data = await get_request_policy(token)

    access_key_id = data["accessKeyId"]
    access_key_secret = data["accessKeySecret"]
    security_token = data["securityToken"]
    bucket_name = data["bucketName"]
    endpoint = data["endpoint"]
    dir = data["dir"]

    # 创建OSS客户端    with timer():
    #         arun(hailuo_upload(file))
    auth = oss2.StsAuth(access_key_id, access_key_secret, security_token)
    bucket = oss2.Bucket(auth, endpoint, bucket_name)

    # file
    size = len(file)

    # 要上传的文件路径和文件名
    mimeType = "png"
    filename = f"{uuid.uuid4()}.{mimeType}"
    object_name = f"{dir}/{filename}"  # png写死
    bucket.put_object(object_name, file)

    params = {}
    payload = {
        "originFileName": "_.png",
        # "originFileName": "503af3b5-9c3b-4bdc-a6d4-256debce3dd5_00001_.png",

        "filename": filename,
        # "fileName": "db884a7c-99a4-40f5-929e-db8769dbf64a.png",

        "dir": dir,
        "endpoint": "oss-cn-wulanchabu.aliyuncs.com",
        "bucketName": "minimax-public-cdn",
        "size": f"{size}",
        "mimeType": mimeType,
        "fileMd5": md5(file),
        "fileScene": 10
    }
    headers = {
        'token': token
    }
    logger.debug(headers)

    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=120) as client:
        response = await client.post("/v1/api/files/policy_callback", params=params, json=payload)
        response.raise_for_status()
        data = response.json()
        logger.debug(bjson(data))
        return data

    # {'data': {'fileID': '307139766909288449',
    #           'ossPath': 'https://cdn.hailuoai.com/prod/2024-10-28-10/user/multi_chat_file/c9d8fc64-d9bf-42c0-8779-f39973579fca.png?image_process=resize,fw_320/format,webp'},
    #  'statusInfo': {'code': 0,
    #                 'debugInfo': '',
    #                 'httpCode': 0,
    #                 'message': '成功',
    #                 'requestID': '33a42a75-15e5-4d4e-b425-eeefc4d3f374',
    #                 'serverAlert': 0,
    #                 'serviceTime': 1730083971}}

async def check_token(token, threshold: int = 30, **kwargs):
    if not isinstance(token, str):
        return await check_tokens(token, check_token)

    try:
        token = await get_access_token(token)
        headers = {
            "token": token
        }
        params = {}
        async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
            response = await client.get("/v1/api/user/equity", params=params)
            response.raise_for_status()
            data = response.json()

            logger.debug(bjson(data))

            return any(i in str(data) for i in {"Ultra", "Unlimited", "高级会员"})
    except Exception as e:
        logger.error(e)
        logger.debug(token)
        return False

if __name__ == '__main__':
    token = "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJleHAiOjE3NzIyODE1NzcsInVzZXIiOnsiaWQiOiI0NDQyMjk2MDAzMzA0OTgwNTUiLCJuYW1lIjoibWZ1aiBiamhuIiwiYXZhdGFyIjoiIiwiZGV2aWNlSUQiOiIzMzkxMTQ5Mjg4NjU1Mjk4NjQiLCJpc0Fub255bW91cyI6ZmFsc2V9fQ.__NDyZQQqyYb7TLrumo944EfuCmrbzYngQloNBK4CmM"

    # arun(get_access_token(token))

    arun(get_request_policy(token))

    # access_token = token
    # arun(upload(Path("img.png").read_bytes(), token=access_token))





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : models
# @Time         : 2025/4/14 11:09
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

"""

[Model(id='LLM-Research/c4ai-command-r-plus-08-2024', created=1725120000, object='model', owned_by='system'),
 Model(id='mistralai/Mistral-Small-Instruct-2409', created=1725120000, object='model', owned_by='system'),
 Model(id='mistralai/Ministral-8B-Instruct-2410', created=1727712000, object='model', owned_by='system'),
 Model(id='mistralai/Mistral-Large-Instruct-2407', created=1719763200, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-Coder-32B-Instruct', created=1731340800, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-Coder-14B-Instruct', created=1731340800, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-Coder-7B-Instruct', created=1731340800, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-72B-Instruct', created=1737907200, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-32B-Instruct', created=1737907200, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-14B-Instruct', created=1737907200, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-7B-Instruct', created=1737907200, object='model', owned_by='system'),
 Model(id='Qwen/QwQ-32B-Preview', created=1737907200, object='model', owned_by='system'),
 Model(id='LLM-Research/Llama-3.3-70B-Instruct', created=1733414400, object='model', owned_by='system'),
 Model(id='opencompass/CompassJudger-1-32B-Instruct', created=1733414400, object='model', owned_by='system'),
 Model(id='Qwen/QVQ-72B-Preview', created=1735056000, object='model', owned_by='system'),
 Model(id='LLM-Research/Meta-Llama-3.1-405B-Instruct', created=1721664000, object='model', owned_by='system'),
 Model(id='LLM-Research/Meta-Llama-3.1-8B-Instruct', created=1721664000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2-VL-7B-Instruct', created=1726675200, object='model', owned_by='system'),
 Model(id='LLM-Research/Meta-Llama-3.1-70B-Instruct', created=1721664000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-14B-Instruct-1M', created=1737907200, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-7B-Instruct-1M', created=1737907200, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-VL-3B-Instruct', created=1737907200, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-VL-7B-Instruct', created=1737907200, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-VL-72B-Instruct', created=1737907200, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-R1-Distill-Llama-70B', created=1737302400, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-R1-Distill-Llama-8B', created=1737302400, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-R1-Distill-Qwen-32B', created=1737302400, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-R1-Distill-Qwen-14B', created=1737302400, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-R1-Distill-Qwen-7B', created=1737302400, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-R1-Distill-Qwen-1.5B', created=1737302400, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-V3', created=1737302400, object='model', owned_by='system'),
 Model(id='Qwen/QwQ-32B', created=1732517497, object='model', owned_by='system'),
 Model(id='XGenerationLab/XiYanSQL-QwenCoder-32B-2412', created=1732517497, object='model', owned_by='system'),
 Model(id='Qwen/Qwen2.5-VL-32B-Instruct', created=1732517497, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-V3-0324', created=1732517497, object='model', owned_by='system'),
 Model(id='Wan-AI/Wan2.1-T2V-1.3B', created=0, object='model', owned_by='system'),
 Model(id='LLM-Research/Llama-4-Scout-17B-16E-Instruct', created=1732517497, object='model', owned_by='system'),
 Model(id='LLM-Research/Llama-4-Maverick-17B-128E-Instruct', created=1732517497, object='model', owned_by='system'),
 Model(id='Qwen/Qwen3-0.6B', created=1745856000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen3-1.7B', created=1745856000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen3-4B', created=1745856000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen3-8B', created=1745856000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen3-14B', created=1745856000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen3-30B-A3B', created=1745856000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen3-32B', created=1745856000, object='model', owned_by='system'),
 Model(id='Qwen/Qwen3-235B-A22B', created=1745856000, object='model', owned_by='system'),
 Model(id='XGenerationLab/XiYanSQL-QwenCoder-32B-2504', created=1732517497, object='model', owned_by='system'),
 Model(id='deepseek-ai/DeepSeek-R1-0528', created=1748361600, object='model', owned_by='system')]
"""

modelscope_model_mapping = {
    "deepseek-reasoner": "deepseek-ai/DeepSeek-R1-0528",
    "deepseek-r1": "deepseek-ai/DeepSeek-R1-0528",
    "deepseek-r1-0528": "deepseek-ai/DeepSeek-R1-0528",
    "deepseek-r1-250528": "deepseek-ai/DeepSeek-R1-0528",

    "deepseek-chat": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3": "deepseek-ai/DeepSeek-V3",
    "deepseek-v3-0324": "deepseek-ai/DeepSeek-V3-0324",
    "deepseek-v3-250324": "deepseek-ai/DeepSeek-V3-0324",

    "majicflus_v1": "MAILAND/majicflus_v1",

}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/12/11 11:03
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.llm.clients import AsyncClient
from meutils.notice.feishu import send_message_for_images
from meutils.io.files_utils import to_url, to_url_fal
from meutils.schemas.image_types import ImageRequest, ImagesResponse

"""

# Set your API key
export API_KEY="<YOUR API Key>"
curl -X POST 'https://api.netmind.ai/v1/generation' \
--header 'Authorization: Bearer ${API_KEY}' \
--header 'Content-Type: application/json' \
--data-raw '{
    "model": "google/nano-banana-pro",
    "config": {
        "prompt": "Use the nano-banana-pro model to create a 1/7 scale commercialized figure of thecharacter in the illustration, in a realistic style and environment. Place the figure on a computer desk, using a circular transparent acrylic base without any text.On the computer screen, display the ZBrush modeling process of the figure. Next to the computer screen, place a BANDAI-style toy packaging box printed with the original artwork.",
        "image_urls": [
            "https://netmind-public-test.s3.us-west-2.amazonaws.com/inference-example-data/fire-dragon.png"
        ],
        "aspect_ratio": "auto",
        "output_format": "png",
        "resolution": "1K" 
    }
}'

{
    "model": "google/nano-banana-pro",
    "config": {
        "prompt": "Use the nano-banana-pro model to create a 1/7 scale commercialized figure of thecharacter in the illustration, in a realistic style and environment. Place the figure on a computer desk, using a circular transparent acrylic base without any text.On the computer screen, display the ZBrush modeling process of the figure. Next to the computer screen, place a BANDAI-style toy packaging box printed with the original artwork.",
        "image_urls": [
            "https://netmind-public-test.s3.us-west-2.amazonaws.com/inference-example-data/fire-dragon.png"
        ],
        "aspect_ratio": "auto",
        "output_format": "png",
        "resolution": "1K",
        "sync_mode": True
    }
}

{'created_at': '2025-12-11 03:11:34',
 'deleted_at': None,
 'id': '943c987abe8d4abdb1e4fdc57a9c4d74',
 'is_deleted': False,
 'logs': [],
 'pending_duration': 34,
 'processing_duration': 0,
 'result': {'data': [{'file_id': 'file-5y5iowc8i6skbfhw',
                      'file_name': 'output_image.png',
                      'file_type': 'image',
                      'url': 'https://files.netmind.ai/ea64296aab884d5bb9896f750ba5e743/inference/file-5y5iowc8i6skbfhw/output_image.png'}]},
 'status': 'completed',
 'updated_at': '2025-12-11 03:12:08',
 'user_id': 'ea64296aab884d5bb9896f750ba5e743'}
 
 
 {'created_at': '2025-12-11 03:18:17',
 'deleted_at': None,
 'id': 'b9fd82de98704e05878b8d59963bc8f4',
 'is_deleted': False,
 'logs': [],
 'result': {},
 'status': 'pending',
 'updated_at': '2025-12-11 03:18:17',
 'user_id': 'ea64296aab884d5bb9896f750ba5e743'}
"""

BASE_URL = "https://api.netmind.ai/v1"


async def generate(request: ImageRequest, api_key: Optional[str] = None):
    client = AsyncClient(base_url=BASE_URL, api_key=api_key)

    payload = {
        "model": request.model,
        "config": {
            "prompt": request.prompt,
            "image_urls": request.image_urls,
            "aspect_ratio": request.aspect_ratio or "auto",
            "output_format": "png",
            "resolution": request.resolution or "2K",  # 0.12 0.2 0.039

            "num_images": request.n or 1,
            # "sync_mode": True,
        }
    }
    if request.model in {"google/nano-banana-pro"}:
        payload["config"]["sync_mode"] = True
        # payload["config"]["resolution"] = request.resolution or "2K"  # 0.12 0.2 0.039
        # payload["config"]["aspect_ratio"] = request.aspect_ratio or "auto"


    # task_id = "df98adfb253341d693f9415bfa00632e"
    # if request.stream:
    #     # "https://api.netmind.ai/v1/generation/{{GENERATION_ID}}"
    #     pass

    response = await client.post(
        "/generation",
        body=payload,
        cast_to=object
    )

    logger.debug(bjson([payload, response]))

    if data := (response.get("result") or {}).get('data', []):
        response = ImagesResponse(data=data)
    # elif task_id := response.get("id"):
    #     for i in range(10):
    #         response = await client.get(f"/generation/{task_id}", cast_to=object)
    #         logger.debug(bjson(response))
    #         if response.get("status") == "completed":
    #             break
    #         await asyncio.sleep(5)
    #     else:
    #         raise TimeoutError(f"任务 {task_id} 超时")
    #     """
    #    "text": "Something error: 422: {\"detail\":[{\"type\":\"literal_error\",\"loc\":[\"body\",\"aspect_ratio\"],\"msg\":\"Input should be '21:9', '16:9', '3:2', '4:3', '5:4', '1:1', '4:5', '3:4', '2:3' or '9:16'\",\"input\":\"auto\",\"ctx\":{\"expected\":\"'21:9', '16:9', '3:2', '4:3', '5:4', '1:1', '4:5', '3:4', '2:3' or '9:16'\"}}]}",
    #
    #     """

    return response


if __name__ == '__main__':
    model = "google/nano-banana"
    # model = "google/nano-banana-pro"

    request = ImageRequest(
        model=model,
        prompt="一个裸体美女",
        # image=["https://netmind-public-test.s3.us-west-2.amazonaws.com/inference-example-data/fire-dragon.png"],

        # image_urls=["https://netmind-public-test.s3.us-west-2.amazonaws.com/inference-example-data/fire-dragon.png"],
        aspect_ratio="auto",
        # resolution="1K",
        n=1,
    )

    arun(generate(request, "b085e2e0b7c149f381f872b8d293f29b"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : audio
# @Time         : 2025/12/11 17:17
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from meutils.schemas.openai_types import TTSRequest
from openai import OpenAI

"""
input: str,
model: Union[str, SpeechModel],
voice: Union[
    str, Literal["alloy", "ash", "ballad", "coral", "echo", "sage", "shimmer", "verse", "marin", "cedar"]
],
instructions: str | Omit = omit,
response_format: Literal["mp3", "opus", "aac", "flac", "wav", "pcm"] | Omit = omit,
speed: float | Omit = omit,
stream_format: Literal["sse", "audio"] | Omit = omit,
# Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
# The extra values given here take precedence over values defined on the client or passed to this method.
extra_headers: Headers | None = None,
extra_query: Query | None = None,
extra_body: Body | None = None,
timeout: float | httpx.Timeout | None | NotGiven = not_given,

{
    "model": "test",
    "input": "你好你好你好你好你好你好你好你好你好你好你好你好你好",
    "voice": "",
    "response_format": "url",
    "chunking_strategy": null,
    "include": null,
    "known_speaker_names": null,
    "known_speaker_references": null,
    "language": "",
    "prompt": "",
    "stream": false,
    "temperature": null,
    "timestamp_granularities": null
}


    
{
    "id": "test",
    "status": "SUBMITTED",
    "headers": {
        "host": "openai-dev.chatfire.cn",
        "x-real-ip": "172.17.0.3",
        "x-forwarded-for": "172.17.0.3",
        "remote-host": "172.17.0.3",
        "connection": "close",
        "content-length": "330",
        "content-type": "application/json",
        "accept": "*/*",
        "authorization": "Bearer https://openai-dev.chatfire.cn/v0",
        "accept-encoding": "gzip",
        "user-agent": "Go-http-client/2.0"
    },
    "url": "http://openai-dev.chatfire.cn/v0/v1/audio/speech",
    "method": "POST",
    "path": "v1/audio/speech",
    "payload": {
        "model": "test",
        "input": "你好你好你好你好你好你好你好你好你好你好你好你好你好",
        "voice": "",
        "response_format": "url",
        "chunking_strategy": null,
        "include": null,
        "known_speaker_names": null,
        "known_speaker_references": null,
        "language": "",
        "prompt": "",
        "stream": false,
        "temperature": null,
        "timestamp_granularities": null
    },
    "form": {},
    "params": {},
    "x-headers": null,
    "model": "test",
    "input": "你好你好你好你好你好你好你好你好你好你好你好你好你好",
    "voice": "",
    "response_format": "url",
    "chunking_strategy": null,
    "include": null,
    "known_speaker_names": null,
    "known_speaker_references": null,
    "language": "",
    "prompt": "",
    "stream": false,
    "temperature": null,
    "timestamp_granularities": null
}
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : uniform_queue
# @Time         : 2023/12/6 17:59
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import time

from meutils.pipe import *


class SmoothQueue(object):
    def __init__(self, end=None):
        self.queue = asyncio.Queue()
        self.end = end

    async def producer(self, generator):  # 生产者将任务放入队列
        if inspect.isasyncgen(generator) or hasattr(generator, '__anext__'):  # if hasattr(future, '__anext__'):
            async for i in generator:
                await self.queue.put(i)
        else:
            for i in generator:
                await self.queue.put(i)

        # 生产结束
        await self.queue.put(self.end)  # 使用 None 作为生产结束的标志

    async def consumer(self, generator, delay: float = 1, debug: bool = False):
        producer_task = asyncio.create_task(self.producer(generator))  # 是否合理？

        for i in range(100000):
            item = await self.queue.get()  # await asyncio.wait_for(self.queue.get(), timeout=2)

            if debug: logger.debug(item)

            if item == self.end: break  # 生产者已经结束生产

            yield item  # 不包含【结束值】

            # 生成器等待，模拟处理速度，这里处理速度是均匀的
            if i > 5: await asyncio.sleep(delay)  # 调整此处的等待时间来控制消费的速度

        await producer_task

    async def main(self, generator):
        producer_task = asyncio.create_task(self.producer(generator))

        async for item in self.consumer(generator):
            print(f'Consumed {item}')

        await producer_task


if __name__ == '__main__':
    async def gen():
        await asyncio.sleep(10)
        for i in range(10):
            logger.debug(time.ctime())
            # 模拟不均匀的生产速度
            await asyncio.sleep(random.random())
            print(f'{time.ctime()}: Produced {i}')
            yield i


    sq = SmoothQueue()


    async def main():
        iters = sq.consumer(gen(), delay=2)
        for i in range(10):
            await asyncio.sleep(1)
            logger.debug(f"{time.ctime()}: {i}")

        async for item in iters:
            print(f'Consumed {item}')

        # await producer_task


    arun(main())

    # import asyncio
    # import random
    #
    #
    # async def producer(queue):
    #     for i in range(10):
    #         # 模拟不均匀的生产速度
    #         await asyncio.sleep(random.random())
    #         await queue.put(i)
    #         print(f'Produced {i}')
    #     # 生产结束
    #     await queue.put(None)  # 使用 None 作为生产结束的标志
    #
    #
    # async def consumer(queue):
    #     while True:
    #         item = await queue.get()
    #         if item is None:
    #             # 生产者已经结束生产
    #             break
    #         # 生成器等待，模拟处理速度，这里处理速度是均匀的
    #         await asyncio.sleep(1)  # 调整此处的等待时间来控制消费的速度
    #         yield item
    #
    #
    # async def main():
    #     queue = asyncio.Queue()
    #     producer_task = asyncio.create_task(producer(queue))
    #
    #     async for item in consumer(queue):
    #         print(f'Consumed {item}')
    #
    #     await producer_task
    #
    #
    # asyncio.run(main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/4/7 13:07
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : D3 生图、编辑图
"""
# {
    #     "index": 0,
    #     "type": "image_url",
    #     "image_url": {
    #         "url": "b64"
    #     }
    #
    # }

"""

from meutils.pipe import *
from meutils.io.files_utils import to_url, to_base64
from meutils.str_utils import parse_base64
from meutils.llm.clients import AsyncOpenAI
from meutils.apis.images.edits import edit_image, ImageProcess

from meutils.schemas.image_types import ImageRequest, ImagesResponse
from meutils.schemas.openai_types import CompletionRequest


async def openai_generate(request: ImageRequest, api_key: Optional[str] = None, base_url: Optional[str] = None):
    api_key = api_key or os.getenv("OPENROUTER_API_KEY")

    is_hd = False
    if request.model.endswith("-hd"):
        is_hd = True
        request.model = request.model.removesuffix("-hd")

    image_urls = request.image_urls
    # image_urls = await to_url(image_urls, filename='.png', content_type="image/png")
    # image_urls = await to_base64(image_urls, content_type="image/png")

    image_urls = [
        {
            "type": "image_url",
            "image_url": {
                "url": image_url
            }
        }
        for image_url in image_urls or []
    ]

    _request = CompletionRequest(
        model=request.model,
        stream=False,
        max_tokens=None,
        messages=[
            {
                "role": "user",
                "content": [
                    {
                        "type": "text",
                        "text": request.prompt
                    },
                    *image_urls
                ]
            }
        ],
    )

    data = _request.model_dump(exclude_none=True)

    client = AsyncOpenAI(
        base_url=base_url,
        api_key=api_key
        # base_url="https://openrouter.ai/api/v1",
        # base_url="https://all.chatfire.cn/openrouter/v1",
        # api_key=api_key or os.getenv("OPENROUTER_API_KEY"),

    )

    completion = await client.chat.completions.create(**data)
    logger.debug(completion)
    if completion and completion.choices and (revised_prompt := completion.choices[0].message.content.strip()):
        logger.debug(revised_prompt)

        if (hasattr(completion.choices[0].message, "images") and (images := completion.choices[0].message.images)):

            image_urls = [image['image_url']['url'] for image in images]
        else:
            image_urls = parse_base64(revised_prompt)

        if is_hd:
            # logger.debug(image_urls)
            tasks = [edit_image(ImageProcess(model="clarity", image=image_url)) for image_url in image_urls]
            responses = await asyncio.gather(*tasks)

            image_urls = [dict(response.data[0])["url"] for response in responses if response.data]
            response = ImagesResponse(image=image_urls)

        else:
            image_urls = await to_url(image_urls, filename=f'{shortuuid.random()}.png', content_type="image/png")
            response = ImagesResponse(image=image_urls)

        # logger.debug(response)

        if response.data:
            return response

    # content_filter
    raise Exception(f"Image generate failed: {completion}")


if __name__ == '__main__':
    # base_url = "https://all.chatfire.cn/openrouter/v1"
    # api_key = os.getenv("OPENROUTER_API_KEY")

    # api_key = "sk-MAZ6SELJVtGNX6jgIcZBKuttsRibaDlAskFAnR7WD6PBSN6M"
    # base_url = "https://new.yunai.link/v1"

    # base_url = "https://api.huandutech.com/v1"
    # api_key = "sk-qOpbMHesasoVgX75ZoeEeBEf1R9dmsUZVAPcu5KkvLFhElrn"

    base_url = "http://209.222.101.251:3014/v1beta"

    api_key = "sk-S6rE16CQyLC2ONtbMO5gdOqkSzDhdWJXAr2oQxMUhxZUV2644"

    request = ImageRequest(
        # model="google/gemini-2.5-flash-image-preview:free",
        # model="google/gemini-2.5-flash-image-preview:free-hd",

        model="gemini-2.5-flash-image",

        # prompt="裸体女孩",

        prompt="a cat",
        # image=["https://oss.ffire.cc/files/kling_watermark.png"],
    )

    r = arun(
        openai_generate(
            request, base_url=base_url, api_key=api_key
        )
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/11/18 16:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo black-forest-labs/flux-1.1-pro-ultra
# https://replicate.com/black-forest-labs/flux-1.1-pro-ultra/api

from meutils.pipe import *
from meutils.schemas.image_types import ImageRequest, ImagesResponse
from meutils.apis.translator import deeplx

import replicate


async def generate(request: ImageRequest, api_key: str = None, base_url: str = None):
    usage = None

    api_key = api_key or os.getenv("REPLICATE_API_KEY")
    client = replicate.client.Client(api_token=api_key)

    payload = {
        "prompt": request.prompt,
        "aspect_ratio": request.aspect_ratio or "match_input_image" if request.image_urls else request.aspect_ratio or "1:1",
        "output_format": request.output_format or "png",
    }
    if request.resolution:
        payload["resolution"] = request.resolution

    if 'z-image' in request.model:
        if request.size:
            payload['width'], payload['height'] = map(int, request.size.split('x'))

        payload['guidance_scale'] = request.guidance or 0
        payload['num_inference_steps'] = request.steps or 8

    elif request.model.startswith(("google/nano-banana",)):
        payload["image_input"] = request.image_urls

    elif request.model.startswith("black-forest-labs/flux-2"):
        payload["go_fast"] = True
        payload["safety_tolerance"] = request.safety_tolerance or 5
        payload["input_images"] = request.image_urls
        if request.resolution:  # pro
            payload["resolution"] = request.resolution.replace('k', ' MP')

        usage = request.usage


    elif request.model.startswith(("flux-kontext",)):
        payload["prompt"] = await deeplx.llm_translate(request.prompt)

        if request.model.endswith("-max"):
            request.model = "black-forest-labs/flux-kontext-max"
        else:
            request.model = "black-forest-labs/flux-kontext-pro"

        if len(request.image_urls) == 1:
            request.model = "black-forest-labs/flux-kontext-dev"  ##########
            payload["input_image"] = request.image_urls[0]

        elif len(request.image_urls) > 1:
            request.model = request.model.replace("black-forest-labs/flux-", "flux-kontext-apps/multi-image-")

            payload["input_image_1"] = request.image_urls[0]
            payload["input_image_2"] = request.image_urls[1]
            # payload["input_image_1"], payload["input_image_2"], *_ = request.image_urls

    logger.debug(request.model)
    logger.debug(bjson(payload))

    output = await client.async_run(
        request.model,
        input=payload
    )

    return ImagesResponse(image=output.url, usage=usage)
    #
    # for i in range(2):
    #     try:
    #         output = await client.async_run(
    #             request.model,
    #             input=payload
    #         )
    #
    #         return ImagesResponse(image=output.url)
    #     except Exception as e:
    #         logger.error(e)
    #         if "Width * Height must be" in str(e):
    #             payload.pop("aspect_ratio", None)


#
# output = client.async_run(
#     "flux-kontext-apps/multi-image-kontext-pro",
#     input={
#         "prompt": "Put the woman next to the house",
#         "aspect_ratio": "match_input_image",
#         "input_image_1": "https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp",
#         "input_image_2": "https://replicate.delivery/pbxt/N7gRAK5kbPwdsbOpqgyAIOFQX45U6suTlbL6ws2N74SnGFpo/test.jpg",
#         "output_format": "png",
#         "safety_tolerance": 2
#     }
# )

# To access the file URL:
# print(output.url)
# => "http://example.com"

if __name__ == '__main__':
    # image_request = ImageRequest(
    #     prompt="Put the woman next to the house",
    #     aspect_ratio="match_input_image",
    #     input_image_1="https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp",
    #     input_image_2="https://replicate.delivery/pbxt/N7gRAK5kbPwdsbOpqgyAIOFQX45U6suTlbL6ws2N74SnGFpo/test.jpg",
    #     output_format="png",
    #     safety_tolerance=2
    # )

    # arun(output)
    model = "flux-kontext-max"
    #     model = "flux-kontext-apps/multi-image-kontext-pro"
    #     model = "flux-kontext-pro"
    #     model = "google/imagen-3"
    #     model = "google/imagen-4"
    #     model = "black-forest-labs/flux-1.1-pro-ultra"
    #     model = "black-forest-labs/flux-2-dev"
    #     model = "black-forest-labs/flux-2-pro"
    #     model = "black-forest-labs/flux-2-pro_4k"
    #     model = "prunaai/z-image-turbo:7ea16386290ff5977c7812e66e462d7ec3954d8e007a8cd18ded3e7d41f5d7cf"
    prompt = """
    手机原相机实拍，无滤镜无修图，33岁素颜少妇(自然皮肤纹理、原生眉形、淡淡唇色，无妆容痕迹)，身材火辣傲人，上围饱满。穿着黑丝透明丝袜，姿态性感又慵懒。表情自然。
穿搭居家性感。随机正在进行式动作。辅导一个18岁女孩作业。
随机家庭日常场景。(自然光，无刻意补光)，轻微过曝，画面颗粒感，随机镜头。构图随意自然，毫无修饰感。 远景/中景。突出生活化抓拍感。时尚摄影。比例 9:16。
    """

    # model = "google/nano-banana-pro_4K"

    request = ImageRequest(
        model=model,
        # prompt="Put the woman next to the house",
        # prompt="一个裸体女人",
        # prompt='带个墨镜',
        prompt=prompt,
        size="2048x2048",

        # aspect_ratio="match_input_image",
        # input_image_1="https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp",
        # input_image_2="https://replicate.delivery/pbxt/N7gRAK5kbPwdsbOpqgyAIOFQX45U6suTlbL6ws2N74SnGFpo/test.jpg",
        # image="https://s3.ffire.cc/files/jimeng.jpg",
        # image="https://replicate.delivery/pbxt/N7gRAUNcVF6HarL0hdAQA2JYNMlJD52LP1wyaIWRUXWeHzqT/0_1-1.webp"
    )
    print(request)

    arun(generate(request))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : usage_types
# @Time         : 2025/11/26 23:26
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from openai.types.completion_usage import CompletionUsage
from openai.types.images_response import UsageInputTokensDetails, Usage as ImageUsage


class Usage(CompletionUsage, ImageUsage):
    # CompletionUsage
    completion_tokens: int = 0
    """Number of tokens in the generated completion."""

    prompt_tokens: int = 0  # text_tokens
    """Number of tokens in the prompt."""

    total_tokens: int = 0
    """Total number of tokens used in the request (prompt + completion)."""

    # ImageUsage
    input_tokens: int = 0
    input_tokens_details: Optional[UsageInputTokensDetails] = None

    output_tokens: int = 0

    # extra
    text_tokens: int = 0
    image_tokens: int = 0

    def __init__(self, /, **data: Any):
        super().__init__(**data)
        # self.prompt_tokens + self.completion_tokens +

        if not self.input_tokens:
            self.input_tokens = self.text_tokens + self.image_tokens

        if not self.input_tokens_details:
            self.input_tokens_details = UsageInputTokensDetails(
                image_tokens=self.image_tokens,
                text_tokens=self.prompt_tokens
            )

        self.prompt_tokens = self.prompt_tokens or self.input_tokens
        self.completion_tokens = self.completion_tokens or self.output_tokens

        self.input_tokens = self.input_tokens or self.prompt_tokens + self.image_tokens
        self.output_tokens = self.output_tokens or self.completion_tokens

        if not self.total_tokens:
            self.total_tokens = self.input_tokens + self.output_tokens


if __name__ == '__main__':
    print(Usage(input_tokens=1, output_tokens=10))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : napkin_types
# @Time         : 2024/12/3 17:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

BASE_URL = "https://nlp-california-api.napkin.ai/api/v1"

ASSETS_BASE_URL = "https://assets.napkin.ai/assets/db"
# https://assets.napkin.ai/assets/db/24px/coffee-mug--food-drinks--24x24.svg

"https://assets.napkin.ai/assets/db/families/lens-circles-v7--family--3.svg"

class IconsSimilarRequest(BaseModel):
    caption: str


class Icon(BaseModel):
    file: str
    list: str
    name: str
    score: float


class IconsSimilarResponse(BaseModel):
    data: List[Icon]
    metadata: Dict = {}

    def __init__(self, /, **data: Any):
        super().__init__(**data)

        for icon in self.data:
            icon.file = f"{ASSETS_BASE_URL}/{icon.list}/{icon.file}.svg"

    class Config:
        extra = "allow"

#
# {'data': [{'file': 'coffee-read--food-drinks--24x24',
#            'list': '24px',
#            'name': 'coffee-read',
#            'score': 0.48007068037986755},
#           {'file': 'coffee-mug--food-drinks--24x24',
#            'list': '24px',
#            'name': 'coffee-mug',
#            'score': 0.4368043541908264},
#           {'file': 'coffee-cup--food-drinks--24x24',
#            'list': '24px',
#            'name': 'coffee',
#            'score': 0.4181402325630188},
#           {'file': 'time-coffee-time-2--interface-essential--24x24',
#            'list': '24px',
#            'name': 'coffee-time-2',
#            'score': 0.408485472202301},
#           {'file': 'coffee-jar--food-drinks--24x24',
#            'list': '24px',
#            'name': 'coffee-jar',
#            'score': 0.4083172082901001},
#           {'file': 'time-coffee-time-3--interface-essential--24x24',
#            'list': '24px',
#            'name': 'coffee-time-3',
#            'score': 0.40657299757003784},
#           {'file': 'coffee-coldbrew-1--food-drinks--24x24',
#            'list': '24px',
#            'name': 'coldbrew',
#            'score': 0.40559959411621094},
#           {'file': 'coffee-straw--food-drinks--24x24',
#            'list': '24px',
#            'name': 'straw',
#            'score': 0.4047240614891052},
#           {'file': 'coffee-to-go--food-drinks--24x24',
#            'list': '24px',
#            'name': 'go',
#            'score': 0.39064520597457886},
#           {'file': 'dating-cup--romance--24x24',
#            'list': '24px',
#            'name': 'dating-cup',
#            'score': 0.38993847370147705}],
#  'metadata': {}}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_videos
# @Time         : 2026/1/27 19:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from meutils.apis.fal.images import check

from meutils.schemas.task_types import TaskResponse
from meutils.schemas.video_types import VideoRequest

from meutils.schemas.fal_types import FEISHU_URL
from meutils.config_utils.lark_utils import get_next_token_for_polling

from fal_client import AsyncClient
from meutils.schemas.video_types import SoraVideoRequest, Video


class Tasks(object):

    def __init__(self, base_url: Optional[str] = None, api_key: str = None):
        self.api_key = api_key
        self.client = AsyncClient(key=api_key)

    async def create(self, request: SoraVideoRequest):
        application = request.model

        payload = {
            "prompt": request.prompt,
            "delete_video": False,
            "aspect_ratio": "auto"
        }

        if request.model.startswith(("fal-ai/sora",)):
            payload["duration"] = min(int(request.seconds or 4), 12)

            if request.resolution:
                payload['resolution'] = request.resolution

            if request.aspect_ratio:
                payload['aspect_ratio'] = request.aspect_ratio

            if images := request.input_reference:
                payload['image_url'] = images[0]

        response = await self.client.submit(
            application=application,
            arguments=payload
        )
        logger.debug(response.__class__.__name__)  # Queued, InProgress, COMPLETED
        logger.debug(response.__dict__)  # Queued, InProgress, COMPLETED

        task_id = f"{application.replace('/', '|')}::{response.request_id}"

        return Video(id=task_id, status=response.__dict__)

    async def get(self, task_id: str, api_key: Optional[str] = None):
        if api_key:
            self.client = AsyncClient(key=api_key)

        application, task_id = task_id.split('::')
        application = application.replace('|', '/')

        response = await self.client.status(application, task_id, with_logs=True)

        logger.debug(response.__class__.__name__)  # Queued, InProgress, COMPLETED
        logger.debug(response.__dict__)  # Queued, InProgress, COMPLETED

        if metrics := response.__dict__.get("metrics"):  # {'inference_time': 17.29231595993042}
            logger.debug(metrics)
            try:
                response = await self.client.result(application, task_id)

            except Exception as e:

                print(str(e))
                """
                [{'loc': ['body', 'aspect_ratio'], 'msg': "unexpected value; permitted: '9:16', '16:9'", 'type': 'value_error.const', 'ctx': {'given': 'auto', 'permitted': ['9:16', '16:9']}}]
                """

        return response


if __name__ == '__main__':
    request = SoraVideoRequest(model="fal-ai/sora-2/text-to-video")

    api_key = os.getenv("FAL_KEY")
    t = Tasks(api_key=api_key)

    # arun(t.create(request))

    task_id = "fal-ai|sora-2|text-to-video::303f48a7-507e-424a-a9ce-2ddaf0bf21b6"
    arun(t.get(task_id, api_key))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat
# @Time         : 2025/12/4 17:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo 增加 pdf  报错走 kimi 兜底 + vl

from meutils.pipe import *
from openai import AsyncOpenAI
from meutils.io.files_utils import to_bytes
from meutils.schemas.openai_types import CompletionRequest

BASE_URL = "https://ai.gitee.com/v1"

"""
支持 .jpg、.jpeg、.png、.gif、.webp 格式，图片分辨率不超过 2048x2048

curl https://ai.gitee.com/v1/images/ocr \
	-X POST \
	-H "Authorization: Bearer XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" \
	-F "image=@path/to/image.webp" \
	-F "model=HunyuanOCR" \
	-F "prompt=检测并识别图片中的文字，将文本坐标格式化输出。"
	
	
curl https://ai.gitee.com/v1/async/documents/parse \
	-X POST \
	-H "X-Failover-Enabled: true" \
	-H "Authorization: Bearer XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" \
	-F "file=@path/to/file.pdf" \
	-F "model=PaddleOCR-VL" \
	-F "include_image=true" \
	-F "include_image_base64=true" \
	-F "output_format=md"
"""


class Completions(object):
    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None,
                 http_url: Optional[Any] = None):
        self.base_url = base_url or BASE_URL
        self.api_key = api_key or os.environ.get("GITEE_API_KEY")

        self.http_url = http_url

    async def create(self, request: CompletionRequest):
        client = AsyncOpenAI(base_url=self.base_url, api_key=self.api_key)

        payload = {
            "model": request.model,
            "prompt": request.last_user_content,
            "output_format": request.response_format or "md"
        }
        # client.files.create()

        extra_headers = {
            "Content-Type": "multipart/form-data",
            # **(extra_headers or {})
        }

        if images := request.last_urls.get("image_url"):
            # logger.debug(await to_bytes(image[0]))
            image = images[0]

            response = await client.post(
                path="/images/ocr",
                body=payload,
                files={
                    "image": await to_bytes(image),
                },
                cast_to=object,
                options={
                    "headers": extra_headers,
                }
            )
            logger.debug(response)
            return response.get("text_result") or ""

        elif files := request.last_urls.get("file_url"): # kimi paddle ocr
            file = files[0]

            payload["include_image"] = True
            payload['include_image_base64'] = True
            response = await client.post(
                path="/images/ocr",
                body=payload,
                files={
                    "file": await to_bytes(file),
                },
                cast_to=object,
                options={
                    "headers": extra_headers,
                }
            )

        # paddle kimi


if __name__ == '__main__':
    request = CompletionRequest(
        model="HunyuanOCR",  # https://s3.ffire.cc/files/pdf_to_markdown.jpg
        # response_format="json",
        messages=[
            {
                "role": "user",
                "content": [
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": "https://s3.ffire.cc/files/pdf_to_markdown.jpg"
                        }
                    },
                    {
                        "type": "text",
                        "text": "描述内容"
                    }
                ]
            }
        ]
    )
    completions = Completions()
    response = arun(completions.create(request))
    logger.debug(response)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/12/3 15:12
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *

# Python 示例代码
import requests
import json

url = "https://api.bizyair.cn/w/v1/webapp/task/openapi/create"
headers = {
    "Content-Type": "application/json",
    "Authorization": "Bearer sk-"
}
data = {
      "web_app_id": 40046,
      "suppress_preview_output": False,
      "input_values": {
        "26:CLIPTextEncode.text": "3D高级感手办，ip形象，动物ip，潮酷版小黄鸭，高挑的个子，，潮酷类型的头身比例，表情冷酷，扁圆的可爱小黄鸭的脸部轮廓，酷帅的小黄鸭造型，身穿宽松嘻哈文化时装搭，潮酷，飒爽，干净背景",
        "19:EmptySD3LatentImage.width": 1024,
        "19:EmptySD3LatentImage.height": 1024,
        "22:KSampler.seed": 1079022223487044
      }
    }

response = requests.post(url, headers=headers, json=data)
result = response.json()
print("生成结果:", result)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2025/12/15 16:55
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 


from meutils.pipe import *
from meutils.io.files_utils import to_bytes, to_url
from meutils.llm.clients import AsyncOpenAI
from meutils.schemas.image_types import ImageRequest

BASE_URL = "https://api.textin.com/ai/service/v1"


async def generate(request: ImageRequest, api_key: Optional[str] = None):
    app_id, secret_code = (api_key or os.getenv("TEXTIN_API_KEY")).split("|")

    logger.debug(f"{app_id, secret_code}")

    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : x
# @Time         : 2024/11/28 19:30
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/fastapi/asyncer

from meutils.pipe import *
import time

import anyio
from asyncer import asyncify, syncify

@asyncify
def do_sync_work(name: str):
    time.sleep(1)
    return f"Hello, {name}"


@syncify
async def do_async_work(name: str):
    await asyncio.sleep(1)
    return f"Hello, a{name}"

async def main():
    message = await asyncify(do_sync_work)(name="World")
    print(message)


if __name__ == '__main__':


    # arun(do_sync_work('xx'))
    arun(do_async_work('xxx'))


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : DeepNN.
# @File         : hashtrick
# @Time         : 2020-04-10 16:54
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 

import hashlib as _hashlib
from functools import lru_cache
from typing import Union

# ME
from meutils.decorators import singleton

"""Java
import java.nio.charset.StandardCharsets
import com.google.common.hash.Hashing.murmur3_32

def hash(key: String = "key", value: String = "value", bins: Int = 10000): Int = {
    val hashValue: Int = murmur3_32.newHasher.putString(f"{key}:{value}", StandardCharsets.UTF_8).hash.asInt
    Math.abs(hashValue) % bins  
  }
"""


def md5(data: Union[str, bytes]):
    if isinstance(data, str):
        data = data.encode('utf8')
    return _hashlib.md5(data).hexdigest()


def murmurhash(key="key", value="value", bins=None, str2md5=True):
    """key:value"""
    string = f"{value}:{key}"
    if str2md5:
        string = md5(string)

    from sklearn.utils.murmurhash import murmurhash3_32 as _murmurhash3_32

    _ = _murmurhash3_32(string, positive=True)  # 与java一致
    return _ % bins if bins else _


@lru_cache()
class ABTest(object):
    """
    https://abtestguide.com/calc/
    http://www.abtestcalculator.com/
    https://zhuanlan.zhihu.com/p/130778873
    https://blog.csdn.net/qq_43656500/article/details/120977558
    https://blog.csdn.net/weixin_43885654/article/details/106750764

    expid = '10001'
    r = {'expid': expid}

    if ABTest((0, 99), **r).is_hit('id'):
        r['data'] = ...
    elif ABTest((100, 199), **r).is_hit('id'):
        r['data'] = ...
    """

    def __init__(self, expid='10001', ranger=(0, 9), bins=100):
        self._bins = bins
        self._ranger = set(range(*ranger))  # 控制流量
        self._expid = expid

    @lru_cache(8 * 10240)  # 8 * 20kb * 长度
    def is_hit(self, value="userid"):
        s = f"{value}:{self._expid}"

        from sklearn.utils.murmurhash import murmurhash3_32 as _murmurhash3_32

        _ = _murmurhash3_32(s, positive=True) % self._bins  # 与java一致
        return _ in self._ranger


if __name__ == '__main__':
    print(md5("key:value"))
    print(murmurhash(str2md5=False, bins=10000))  # 3788
    print(murmurhash(str2md5=True, bins=10000))  # 1688 5608
    expid = '10001'
    r = {'expid': expid}

    print(ABTest(**r, ranger=(0, 99)).is_hit('id'))
    r = {'expid': 'expid'}

    print(ABTest(**r, ranger=(0, 99)).is_hit('id'))


    import uuid
    print(uuid.uuid3('1', '1'))


import json
import traceback
from functools import partial
from httpx import HTTPStatusError
from openai import APIStatusError

from fastapi import status
from starlette.requests import Request
from starlette.responses import JSONResponse

from fastapi.exceptions import RequestValidationError, HTTPException

from meutils.notice.feishu import send_message_for_http as send_message

exc_set = {

    # gemini
    "IMAGE_PROHIBITED_CONTENT",
    "IMAGE_SAFETY",
    "Provided image is not valid",

    "Text Risk Not Pass",

    "The input or output was flagged as sensitive",
    "invalidReferenceImageSize",
    "string_too_long",

    "content moderation",
    "please check the prompt",

    "sensitive",
    "OversizeImage",
    "filtered by safety checks",
    "InvalidParameter",
    "Image size format should",
    "Content security",

    "Failed to load the image",
    "Timeout while downloading url",

    "not corrupted",

    "supported format",
    "format unsupported",
    "UnsupportedImageFormat",

    "invalid image",
    "Image Decode Error",

    "Image pixel is invalid",

    "cannot identify",
    "could not generate",

    "Image size format should be like",

    # 大小/分辨率
    "Image resolution exceeds maximum",
    "Image dimensions out of range",
    "Total pixel count overflow",
    "Image aspect ratio not allowed",
    # 体积/上传
    "Payload Too Large",
    "413 Request Entity Too Large",
    "File size exceeds upload limit",
    # 格式/编码
    "Input buffer contains unsupported image format",
    "detected content type",
    "File extension does not match MIME type",
    "Codec not available for this format",
    # 内容安全
    "Adult content detected",
    "Violence risk score above threshold",
    "Copyright fingerprint hit",
    # 解码/像素
    "Corrupt JPEG data: premature end of data segment",
    "Invalid JPEG marker",
    "PNG signature not found",
    "Unsupported color type",
    "VipsJpeg: out of order read",
    # 网络/存储
    "Connection reset while downloading image",
    "SSL handshake failed",
    "403 Forbidden",
    "S3 AccessDenied",
    "Read timed out",
    # 内存/限流
    "Image processing memory limit exceeded",
    "GPU out of memory",
    "Rate limit exceeded",
    "Concurrent generation limit reached",
    # 参数
    "Missing required field",
    "Invalid base64 padding",
    "Empty image data",
}

exc_replace_set = {
    "runware",
    "fal.ai",
    "api.ppinfra"
}


async def http_error_handler(_: Request, exc: HTTPException) -> JSONResponse:
    # print(exc)
    content = {
        "error":
            {
                "message": f"{exc.detail}",
                "type": "http-error",
            }
    }
    return JSONResponse(
        content=content,
        status_code=exc.status_code
    )


async def general_exception_handler(request: Request, exc: Exception):
    return JSONResponse(
        content={"message": str(exc)},
        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
    )


# todo videos
"""
400 {"title":"Bad Request","status":400,"message":"Invalid payload provided","instance":"/v2/video/generations?","timestamp":"2026-01-22T07:20:46.273Z","error":{"name":"BadRequestException","message":"Invalid payload provided","data":[{"received":"768P","code":"invalid_enum_value","options":["360p","540p","720p","1080p"],"path":["resolution"],"message":"Invalid enum value. Expected '360p' | '540p' | '720p' | '1080p', received '768P'"},{"received":"4","code":"invalid_enum_value","options":["5","8","10"],"path":["duration"],"message":"Invalid enum value. Expected '5' | '8' | '10', received '4'"}]}}
"""
"""oneapi kling o1 居然可以返回
{
    "message": "{\"code\":402,\"msg\":\"您的余额不足，请前往充值中心进行充值\",\"data\":null}",
    "data": {
        "code": 402,
        "error": {
            "message": "{\"code\":402,\"msg\":\"您的余额不足，请前往充值中心进行充值\",\"data\":null}",
            "type": "cf-api-error"
        }
    }
}

{
"message": e1["error"]["message"],

"data":e1
}




e1 = {
    "error": {
        "message": "{\"code\":402,\"msg\":\"您的余额不足，请前往充值中心进行充值\",\"data\":null}",
        "type": "cf-api-error"
    },
    "code": 402
}
"""


async def chatfire_api_exception_handler(request: Request, exc: Exception):
    content = {
        "error":
            {
                "message": f"{exc}",
                "type": "cf-api-error",
            },

        "code": status.HTTP_500_INTERNAL_SERVER_ERROR  # 默认 是否默认都不重试？
    }

    # 默认值
    reps = None
    if isinstance(exc, (HTTPStatusError, APIStatusError)):  # todo 透传状态码
        status_code = exc.response.status_code or 500

        content['code'] = status_code
        if (message := exc.response.text.strip()) and message.startswith("{"):  # 解析内层 code message
            _ = json.loads(message)
            content['code'] = _.get("code") or content['code']
            message = _.get("message") or _.get("msg") or message

        content['error']['message'] = message

        # 置换错误
        for to_replace in exc_replace_set:
            content['error']['message'] = content['error']['message'].replace(to_replace, '***')

        reps = JSONResponse(
            content=content,
            status_code=status_code,
        )

    if any(i.lower() in str(exc).lower() for i in {"Invalid Token"}):  # 不跳过重试
        reps = JSONResponse(
            content=content,
            status_code=status.HTTP_401_UNAUTHORIZED,
        )

    elif any(i.lower() in str(exc).lower() for i in exc_set):  # 跳过重试
        reps = JSONResponse(
            content=content,
            status_code=status.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS,
        )

        # return reps

    # send_message
    content_detail = f"{traceback.format_exc()}"  # 是不是最后几行就可以了
    #
    # from meutils.pipe import logger
    # logger.debug(content_detail)

    if any(code in content_detail for code in {'451', }):
        content_detail = ""

    send_message([content, content_detail], title=__name__)

    # from meutils.pipe import logger
    #
    # try:
    #
    #     logger.debug(request.headers)
    #     logger.debug(request.url)
    #     logger.debug(request.method)
    #     logger.debug(request.query_params._dict)
    #
    #     logger.debug(request.client)
    #     logger.debug(request.cookies)
    #
    #     payload  = await request.body()
    #
    #     # send_message(payload, title=__name__)
    #
    #     logger.debug(payload)
    #     logger.debug(payload)
    #
    #
    # except Exception as e:
    #     logger.debug(e)

    return reps or JSONResponse(
        content=content,
        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
    )


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : faceswap
# @Time         : 2024/8/14 14:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.decorators.retry import retrying

from meutils.schemas.task_types import TaskType, Task
from meutils.schemas.prodia_types import BASE_URL, FEISHU_URL, FaceswapRequest

from meutils.config_utils.lark_utils import get_next_token_for_polling
from meutils.notice.feishu import send_message as _send_message

send_message = partial(
    _send_message,
    title=__name__,
    url="https://open.feishu.cn/open-apis/bot/v2/hook/dc1eda96-348e-4cb5-9c7c-2d87d584ca18"
)


@retrying(max_retries=8, max=8, predicate=lambda r: r is True)  # 触发重试
async def create_task(request: FaceswapRequest, token: Optional[str] = None):
    token = token or await get_next_token_for_polling(FEISHU_URL)

    payload = request.model_dump()

    headers = {
        "X-Prodia-Key": token
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=60) as client:
        response = await client.post("/faceswap", json=payload)
        logger.debug(response.text)

        if response.status_code in {429}:  # 触发重试
            return True

        if response.is_success:
            data = response.json()
            task_id = f"{TaskType.faceswap}-{data['job']}"
            return Task(id=task_id, data=data, system_fingerprint=token)
        else:
            logger.debug(token)
            return Task(data=response.text, status=0, status_code=response.status_code)


async def get_task(task_id: str, token: str):
    task_id = task_id.split(f"{TaskType.faceswap}-", 1)[-1]

    headers = {
        "X-Prodia-Key": token
    }
    async with httpx.AsyncClient(base_url=BASE_URL, headers=headers, timeout=100) as client:
        response = await client.get(f"/job/{task_id}")

        logger.debug(response.text)
        logger.debug(response.status_code)

        if response.is_success:
            data = response.json()
            return data


if __name__ == '__main__':
    request = FaceswapRequest(
        sourceUrl='https://oss.ffire.cc/files/source.jpg',
        targetUrl='https://oss.ffire.cc/files/target.jpg'
    )
    arun(create_task(request))

    arun(get_task('e40e290f-71b6-4755-a4e8-d5201462d9d4', 'fbb99c65-3ac9-4077-8a3c-67387442f48b'))



from typing import List

from meutils.pipe import *

patterns = [
    r'([。!?\?])([^"\'])',
    r'(\.{6})([^"\'])',
    r'(\.{3,}[^\.\s])',
    r'([。!?\?]["\'])([^,。!?\?])'
]
combined_pattern = '|'.join(patterns)


def split_text(text_input: str, chunk_size: int=500, overlap_ratio: float=0.2) -> List[str]:

    # 连续标点句子去重
    sentences = re.sub(combined_pattern, r'\1\n\2', text_input).strip().split("\n")
    sentences_lengths = [len(sentence) for sentence in sentences]
    chunks = []
    fast = 0
    slow = 0
    current_length = 0
    overlap_length = chunk_size * overlap_ratio // 1
    # 防止 overlap_length 过大而 chunk_size 过小导致无限循环
    iter_count = 0
    max_iter_count = len(sentences_lengths) * 2
    while iter_count < max_iter_count and fast < len(sentences_lengths):
        current_length += sentences_lengths[fast]
        if current_length > chunk_size:
            chunk = " ".join(sentences[slow:fast])
            chunks.append(chunk)
            current_length = current_length - sentences_lengths[fast]
            while slow < fast and current_length > overlap_length:
                current_length -= sentences_lengths[slow]
                slow += 1
            fast = slow
            current_length = 0
        else:
            fast += 1
        iter_count += 1
    if iter_count >= max_iter_count:
        raise ValueError('Warning: maximum iteration reached, maybe reduce the overlap_ratio')
    if fast < len(sentences_lengths):
        chunk = " ".join(sentences[fast:])
        chunks.append(chunk)
    return chunks



text = """
中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告2023年03月31日基金管理人:中欧基金管理有限公司基金托管人:中国建设银行股份有限公司报告送出日期:2023年04月22日中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告§1 重要提示基金管理人的董事会及董事保证本报告所载资料不存在虚假记载、误导性陈述或重大遗漏，并对其内容的真实性、准确性和完整性承担个别及连带责任。基金托管人中国建设银行股份有限公司根据本基金合同规定，于2023年4月21日复核了本报告中的财务指标、净值表现和投资组合报告等内容，保证复核内容不存在虚假记载、误导性陈述或者重大遗漏。基金管理人承诺以诚实信用、勤勉尽责的原则管理和运用基金资产，但不保证基金一定盈利。基金的过往业绩并不代表其未来表现。投资有风险，投资者在作出投资决策前应仔细阅读本基金的招募说明书。本报告中财务资料未经审计。本报告期自2023年01月01日起至2023年03月31日止。§2 基金产品概况第2页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告§3 主要财务指标和基金净值表现3.1 主要财务指标单位：人民币元注：1.本期已实现收益指基金本期利息收入、投资收益、其他收入（不含公允价值变动收益）扣除相关费用和信用减值损失后的余额，本期利润为本期已实现收益加上本期公允价值变动收益。2.所述基金业绩指标不包括持有人认购或交易基金的各项费用，计入费用后实际收益水平要低于所列数字。3.2 基金净值表现3.2.1 基金份额净值增长率及其与同期业绩比较基准收益率的比较第3页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告中欧新蓝筹混合A净值表现中欧新蓝筹混合E净值表现中欧新蓝筹混合C净值表现第4页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告3.2.2 自基金合同生效以来基金累计净值增长率变动及其与同期业绩比较基准收益率变动的比较第5页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告注：本基金于2015年10月 8日新增 E类份额，图示日期为2015年10月12日至2023年03月31日。注：本基金于2017年 1月12日新增 C类份额，图示日期为2017年 1月13日至2023年03月31日。§4 管理人报告4.1 基金经理（或基金经理小组）简介注：1、任职日期和离任日期一般情况下指公司作出决定之日；若该基金经理自基金合同生效日起即任职，则任职日期为基金合同生效日。第6页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告2、证券从业的含义遵从《证券基金经营机构董事、监事、高级管理人员及从业人员监督管理办法》等相关规定。4.2 管理人对报告期内本基金运作遵规守信情况的说明本报告期内，本基金管理人严格遵循了《证券投资基金法》及其各项实施细则、本基金基金合同和其他相关法律法规的规定，本着诚实信用、勤勉尽责、取信于市场、取信于社会的原则管理和运用基金资产，为基金份额持有人谋求最大利益。本报告期内，基金投资管理符合有关法规和基金合同的规定，无违法违规、未履行基金合同承诺或损害基金份额持有人利益的行为。4.3 公平交易专项说明4.3.1 公平交易制度的执行情况本报告期内，本基金管理人严格按照《证券投资基金管理公司公平交易制度指导意见》及公司内部相关制度等规定，从研究分析、投资决策、交易执行、事后监控等环节严格把关，通过系统和人工等方式在各个环节严格控制交易公平执行。本报告期内，本基金管理人公平交易制度和控制方法总体执行情况良好，不同投资组合之间不存在非公平交易或利益输送的情况。4.3.2 异常交易行为的专项说明本报告期内，公司旗下所有投资组合参与的交易所公开竞价交易中，同日反向交易成交较少的单边交易量超过该证券当日成交量的5%的交易共有26次，为量化策略组合因投资策略需要发生的反向交易，公司内部风控对上述交易均履行相应控制程序。本报告期内，未发现本基金有可能导致不公平交易和利益输送的异常交易。4.4 报告期内基金的投资策略和运作分析一季度市场以春节为分水岭呈现先扬后抑走势，但总体各指数呈现不同程度的上涨。春节前市场对疫后的经济复苏展望较为乐观，叠加中国经济前景较海外更为确定，因此催生北上资金持续流入A股，市场呈现单边上行走势。节后随着经济和行业的高频数据出台，以及两会后全年经济增长目标逐步明朗，市场对原预期进行修正，股市走弱，震荡回调。从行业来说，与经济总量相关的消费、周期等行业表现不佳，新能源相关行业也因为短期景气度下降表现差强人意。相对而言，由于对中国特色估值体系的认同度不断提升，部分行业的央国企价值得到重估，走势明显偏强。此外，随着ChatGPT的横空出世，海外大模型以及相关人工智能的产业趋势受到市场关注，并很快扩散到国内产业界，市场聚焦于上游算力、传输和下游运用，在经济复苏相对温和的情景下，计算机、通讯、传媒、电子行业受到各方资金的追捧，涨幅遥遥领先。报告期内，本基金基本维持年初组合，相对市场继续超配了养殖与疫情受损股，我们认为今年甚至明年这两个板块的公司盈利明显好转是大概率事件。在宏观经济恢复没那么强劲的背景下，这种不低于市场预期的确定性业绩高增长是比较稀缺的。另外，基第7页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告金增加了处于周期底部且估值合理的化工行业以及受益于数字经济建设的通讯行业的配置，降低了弹性欠佳的部分交运、电力设备和医药行业的权重。4.5 报告期内基金的业绩表现本报告期内，A类基金份额净值增长率为1.42%，同期业绩比较基准收益率为3.08%；E类份额净值增长率为1.42%，同期业绩比较基准收益率为3.08%；C类份额净值 增长率为1.22%，同期业绩比较基准收益率为3.08%。4.6 报告期内基金持有人数或基金资产净值预警说明本报告期内基金管理人无应说明预警信息。§5 投资组合报告5.1 报告期末基金资产组合情况5.2 报告期末按行业分类的股票投资组合5.2.1 报告期末按行业分类的境内股票投资组合第8页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告5.2.2 报告期末按行业分类的港股通投资股票投资组合本基金本报告期末未持有港股通股票。5.3 报告期末按公允价值占基金资产净值比例大小排序的前十名股票投资明细第9页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告5.4 报告期末按债券品种分类的债券投资组合5.5 报告期末按公允价值占基金资产净值比例大小排序的前五名债券投资明细5.6 报告期末按公允价值占基金资产净值比例大小排序的前十名资产支持证券投资明细本基金本报告期末未持有资产支持证券。5.7 报告期末按公允价值占基金资产净值比例大小排序的前五名贵金属投资明细第10页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告本基金本报告期末未持有贵金属。5.8 报告期末按公允价值占基金资产净值比例大小排序的前五名权证投资明细本基金本报告期末未持有权证。5.9 报告期末本基金投资的股指期货交易情况说明5.9.1 报告期末本基金投资的股指期货持仓和损益明细股指期货不属于本基金的投资范围，故此项不适用。5.9.2 本基金投资股指期货的投资政策股指期货不属于本基金的投资范围，故此项不适用。5.10 报告期末本基金投资的国债期货交易情况说明5.10.1 本期国债期货投资政策国债期货不属于本基金的投资范围，故此项不适用。5.10.2 报告期末本基金投资的国债期货持仓和损益明细国债期货不属于本基金的投资范围，故此项不适用。5.10.3 本期国债期货投资评价国债期货不属于本基金的投资范围，故此项不适用。5.11 投资组合报告附注5.11.1 本基金投资的春秋航空的发行主体春秋航空股份有限公司于2022年12月05日受到中国民用航空华北地区管理局的华北局罚决河北字〔2022〕2号，于2022年11月18日受到中国民用航空中南地区管理局的中南局罚决广东字〔2022〕19号。 本基金对上述主体发行的相关证券的投资决策程序符合相关法律法规及基金合同的要求。其余前十大持有证券的发行主体本报告期内没有被监管部门立案调查，或在报告编制日前一年内受到公开谴责、处罚的情形。5.11.2 本基金投资的前十名股票中，没有投资于超出基金合同规定备选库之外的股票。5.11.3 其他资产构成第11页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告5.11.4 报告期末持有的处于转股期的可转换债券明细5.11.5 报告期末前十名股票中存在流通受限情况的说明5.11.6 投资组合报告附注的其他文字描述部分本报告中因四舍五入原因，投资组合报告中市值占总资产或净资产比例的分项之和与合计可能存在尾差。§6 开放式基金份额变动单位：份第12页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告注：总申购份额含红利再投、转换入份额，总赎回份额含转换出份额。§7 基金管理人运用固有资金投资本基金情况7.1 基金管理人持有本基金份额变动情况本基金管理人本报告期内未持有本基金。7.2 基金管理人运用固有资金投资本基金交易明细本基金管理人本报告期内无申购、赎回本基金的情况。§8 影响投资者决策的其他重要信息8.1 报告期内单一投资者持有基金份额比例达到或超过20%的情况报告期内单一投资者持有基金份额比例不存在达到或超过20%的情况。8.2 影响投资者决策的其他重要信息无。§9 备查文件目录9.1 备查文件目录1、中欧新蓝筹灵活配置混合型证券投资基金相关批准文件2、《中欧新蓝筹灵活配置混合型证券投资基金基金合同》3、《中欧新蓝筹灵活配置混合型证券投资基金托管协议》4、《中欧新蓝筹灵活配置混合型证券投资基金招募说明书》5、基金管理人业务资格批件、营业执照6、本报告期内在中国证监会指定媒介上公开披露的各项公告第13页中欧新蓝筹灵活配置混合型证券投资基金2023年第1季度报告9.2 存放地点基金管理人的办公场所。9.3 查阅方式投资者可登录基金管理人网站(www.zofund.com)查阅，或在营业时间内至基金管理人办公场所免费查阅。投资者对本报告书如有疑问，可咨询本基金管理人中欧基金管理有限公司：客户服务中心电话：021-68609700，400-700-9700中欧基金管理有限公司2023年04月22日第14页
"""
res = split_text(text, 500, 0.2)
for i in res:
    print("=" * 20)
    print(i)
    print(len(i))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : common
# @Time         : 2020/11/12 11:42 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 单向引用，避免循环引用

import io
import os
import gc
import re
import sys
import time
import copy
import types
import typing
import uuid
import zipfile
import datetime
import operator
import inspect
import textwrap
import socket
import logging
import tempfile
import warnings

warnings.filterwarnings("ignore")

import functools
from functools import wraps

import argparse
import traceback
import threading
import multiprocessing
import base64
import shutil
import random
import asyncio
import importlib
import itertools
import pickle
import textwrap
import subprocess
import wget
import toml
import yaml
import typer
import json
import mimetypes
import joblib
from joblib.hashing import hash
import httpx
import requests
import wrapt
import shortuuid
# import sklearn  #######
import numpy as np
import pandas as pd

from typing import *
from pathlib import Path
from queue import Queue
from pprint import pprint
from abc import abstractmethod
from dataclasses import dataclass
from urllib.parse import unquote, unquote_plus, urlparse, urljoin
from functools import reduce, lru_cache, partial

from collections import Counter, OrderedDict
from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor

# import matplotlib.pyplot as plt  #######
# from matplotlib.font_manager import FontProperties  #######
#
# FONT = FontProperties(fname=get_resolve_path('./data/SimHei.ttf', __file__))

# matplotlib.use('Agg'): matplotlib 这个库，这个库在使用时会创建额外的线程。如果你的应用不需要使用到 matplotlib，你可以考虑移除这个库，这可能能够解决问题。如果你的应用需要使用 matplotlib，但是并不需要它的图形界面功能，你可以尝试在导入 matplotlib 之前设置 matplotlib 的后端为 'Agg'
# plt.rcParams['axes.unicode_minus'] = False

from loguru import logger
# logger.remove()
# logger.add(sys.stderr, format='<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | <level>{level: <4}</level> - <level>{message}</level>')


from tqdm.auto import tqdm

tqdm.pandas()

from pydantic import BaseModel, Field
from faker import Faker  # https://www.cnblogs.com/aichixigua12/p/13236092.html

fake_zh = Faker(locale='zh_CN')

# from PIL import Image, ImageGrab
# image
# im = ImageGrab.grabclipboard() 获取剪切板的图片
# json
import jsonpath
import json_repair

# 第三方
from meutils.other.crontab import CronTab
from meutils.other.besttable import Besttable

# ME
from meutils._utils import *
from meutils.init.evn import *
from meutils.init.oo import __O000OO0O0000OO00O
from meutils.hash_utils import murmurhash
from meutils.cache_utils import ttl_cache, disk_cache, diskcache
from meutils.decorators import decorator, args, singleton, timer, background, background_task
from meutils.path_utils import get_module_path, get_resolve_path, sys_path_append, path2list, get_config

lock = threading.Lock()
__O000OO0O0000OO00O()

from dotenv import load_dotenv

load_dotenvs = lambda dotenv_paths: [load_dotenv(p, verbose=True) for p in dotenv_paths]

EVN = os.getenv('EVN', "/Users/betterme/PycharmProjects/AI/.env")
load_dotenv(
    EVN,  # EVN=传入绝对路径 todo: 自定义 .env.secret
    verbose=True
)

cli = typer.Typer(name="MeUtils CLI")

# 常量
CPU_NUM = os.cpu_count()

HOST_NAME = DOMAIN_NAME = LOCAL_HOST = LOCAL = HOST = PORT = ''

HOME_CACHE = Path.home() / ".cache"

try:
    if not hasattr(typing, 'Literal'):
        import typing
        import typing_extensions

        Literal = typing_extensions.Literal
        typing.Literal = Literal

    if not hasattr(functools, 'cached_property'):
        from cached_property import cached_property
    else:
        from functools import cached_property

    from IPython.core.interactiveshell import InteractiveShell

    InteractiveShell.ast_node_interactivity = "all"  # 多行输出

    from rich import print as rprint

    HOST_NAME = socket.gethostname()
    DOMAIN_NAME = socket.getfqdn(HOST_NAME)

    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as _st:
        _st.connect(('10.255.255.255', 1))
        HOST, PORT = _st.getsockname()

    # import orjson as json  # dumps 结果是字节型
    # json.dumps = partial(json.dumps, option=json.OPT_NON_STR_KEYS)

    from icecream import ic

    ic.configureOutput(includeContext=True)

except Exception as e:
    pass


def download(url, filename: Optional[str] = None):
    filename = filename or Path(url).name

    with httpx.Client(follow_redirects=True, timeout=100) as client:
        response = client.get(url)
        with open(filename, 'wb') as f:
            f.write(response.content)
        return filename


class BaseConfig(BaseModel):
    """基础配置"""
    _path: str = None

    @classmethod
    def init(cls):
        """init from path[zk/yaml]"""
        assert cls._path is not None, "请指定 _path"
        return cls.parse_path(cls._path)

    @classmethod
    def parse_path(cls, path):
        if Path(path).is_file():
            return cls.parse_yaml(cls._path)
        else:
            return cls.parse_zk(cls._path)

    @classmethod
    def parse_yaml(cls, path):
        json = yaml.safe_load(Path(path).read_bytes())
        return cls.parse_obj(json)

    @classmethod
    def parse_zk(cls, path):
        from meutils.zk_utils import get_zk_config
        json = get_zk_config(path)
        return cls.parse_obj(json)

    @classmethod
    def parse_env(cls):
        return cls.parse_obj(os.environ)


# limit memory
def limit_memory(memory=16):
    """
    :param memory: 默认限制内存为 16G
    :return:
    """
    import resource

    rsrc = resource.RLIMIT_AS
    # res_mem=os.environ["RESOURCE_MEM"]
    memlimit = memory * 1024 ** 3
    resource.setrlimit(rsrc, (memlimit, memlimit))
    # soft, hard = resource.getrlimit(rsrc)
    logger.info("memory limit as: %s G" % memory)


def magic_cmd(cmd='ls', parse_fn=lambda s: s, print_output=False):
    """

    :param cmd:
    :param parse_fn: lambda s: s.split('\n')
    :param print_output:
    :return:
    """
    cmd = ' '.join(cmd.split())
    status, output = subprocess.getstatusoutput(cmd)
    output = output.strip()

    logger.info(f"CMD: {cmd}")
    logger.info(f"CMD Status: {status}")

    if print_output:
        logger.info(f"CMD Output: {output}")

    return status, parse_fn(output)


def run_command(command='ls'):
    """
    运行Shell命令并输出其输出
    """
    process = subprocess.Popen(
        command, stdout=subprocess.PIPE, shell=True, universal_newlines=True
    )
    while True:
        # 读取Shell命令的输出
        key = str(time.time())
        output = process.stdout.readline()
        if output == "" and process.poll() is not None:
            break
        if output:
            # 实时显示输出
            print(output.strip())

    return process.poll()


def is_open(ip='88.01.012.01'[::-1], port=7000, timeout=0.5):
    """
        互联网 is_open('baidu.com:80')

    @param ip:
    @param port:
    @param timeout:
    @return:
    """
    if ':' in ip:
        ip, port = ip.split(':')

    socket.setdefaulttimeout(timeout)
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        try:
            s.connect((ip, int(port)))  # is_open("da.gd", 80) is_open("www.da.gd", 443)
            s.shutdown(socket.SHUT_RDWR)
            return True
        except:
            return False


def get_var_name(var):
    """获取变量字符串名
        a=1
        b=1
        c=1
        只会取 a, 因是通过 id 确定 key
    """
    _locals = sys._getframe(1).f_locals
    for k, v in _locals.items():
        if id(var) == id(v):  # 相同值可能有误判
            return k


def get_current_fn():
    """获取执行函数的函数名
        def f(): # f
            print(get_current_fn())
    @return:
    """
    # inspect.currentframe().f_back === sys._getframe().f_back
    # f_name = inspect.getframeinfo(inspect.currentframe().f_back)[2]  # 最外层

    f_name = sys._getframe(1).f_code.co_name  # sys._getframe(1) 外层 sys._getframe(0) 内层
    return f_name


@lru_cache()
def get_function_params(fn: Optional[Callable] = None):
    if fn is None:
        import openai
        fn = openai.OpenAI(api_key='').chat.completions.create

    params = inspect.signature(fn).parameters
    return list(params.keys())


def clear(ignore=('TYPE_CHECKING', 'logger', 'START_TIME', 'CPU_NUM', 'HOST_NAME', 'LOCAL_HOST', 'LOCAL')):
    """销毁全局变量
    TODO：可添加过滤规则
    """
    keys = []
    ignore = set(ignore)
    for key, value in globals().items():
        if key.startswith('_') or key in ignore:
            continue
        if callable(value) or value.__class__.__name__ == "module":
            continue
        keys.append(key)

    logger.debug("销毁全局变量: " + list4log(keys))
    for key in keys:
        del globals()[key]
    return keys


def show_code(func):
    sourcelines, _ = inspect.getsourcelines(func)
    _ = textwrap.dedent("".join(sourcelines))
    print(_)
    return _


def file_replace(file, old, new):
    p = Path(file)
    _ = (
        p.read_text()
        .replace(old, new)
    )
    p.write_text(_)


def exec_callback(source, **namespace):
    """

    @param source:
    @param namespace: source 入参
    @return: 出参
    """
    namespace = namespace or {}
    exec(source, namespace)
    namespace.pop('__builtins__')
    return namespace  # output


def pkl_dump(obj, file):
    with lock:
        try:
            with open(file, 'wb') as f:
                return pickle.dump(obj, f)
        except IOError:
            return False


def pkl_load(file):
    try:
        with open(file, 'rb') as f:
            return pickle.load(f)
    except IOError:
        return False


class MeBackgroundTasks(object):
    """
    def func(x):
        print(f'Sleeping: {x}')
        time.sleep(x)
        print(f'DONE: {x}')

    bk = BackgroundTasks()
    bk.add_task(func, x=1)
    bk.add_task(func, x=2)
    bk.add_task(func, x=3)
    """

    def __init__(self, max_workers=None, thread_name_prefix='🐶BackgroundTasks'):
        self.pool = ThreadPoolExecutor(max_workers, thread_name_prefix)

    def add_task(self, func, *args, **kwargs):
        future = self.pool.submit(func, *args, **kwargs)  # pool.map(fn, *iterables, timeout=None, chunksize=1)
        future.add_done_callback(lambda x: logger.error(future.exception()) if future.exception() else None)


background_tasks = MeBackgroundTasks()


# import uuid
# uuid.uuid4().hex
# attrs = [attr for attr in dir(i) if not callable(getattr(i, attr)) and not attr.startswith("__")]


def try_import(
        module_name: str, *, pip_name: Optional[str] = None, package: Optional[str] = None
) -> Any:
    """Dynamically imports a module and raises a helpful exception if the module is not
    installed."""
    module = None
    try:
        module = importlib.import_module(module_name, package)
    except ImportError:
        raise ImportError(
            f"Could not import {module_name} python package. "
            f"Please install it with `pip install {pip_name or module_name}`."
        )
    return module


def obj_to_dict(obj):
    """类对象转字典"""
    if isinstance(obj, list):
        return [obj_to_dict(item) for item in obj]
    elif isinstance(obj, dict):
        return {k: obj_to_dict(v) for k, v in obj.items()}
    elif hasattr(obj, '__dict__'):
        return obj_to_dict(obj.__dict__)
    else:
        return obj


def dict_to_model(data: Dict[str, Any], model_name: str = 'DynamicModel'):
    s = f"""class {model_name}(BaseModel):"""
    for k, v in data.items():
        _type = type(v).__name__
        if isinstance(_type, str):
            v = f"'{v}'"
        s += f"\n\t{k}: {_type} = {v}"
        print(s)

    from pydantic import create_model

    # 动态创建模型类
    model_fields = {}
    for key, value in data.items():
        if isinstance(value, dict):
            model_fields[key] = (dict_to_model(value), ...)
        elif isinstance(value, list) and value:
            if isinstance(value[0], dict):
                model_fields[key] = (List[dict_to_model(value[0])], ...)
            else:
                model_fields[key] = (List[type(value[0])], ...)
        else:
            model_fields[key] = (type(value), ...)

    model = create_model(model_name, **model_fields)

    # 创建模型对象
    model_obj = model(**data)

    return model_obj


def _hot_reload(*paths):
    """可以是文件夹"""

    from watchfiles import watch

    for changes in watch(*paths):
        logger.debug(changes)

        file = list(changes)[0][1]

        try:
            if file.endswith('env') or file.startswith('env.'):  # todo: 可增加其他条件
                load_dotenv(file, override=True)  # 覆盖
            elif file.endswith('.toml'):
                os.environ['TOML_CONFIG'] = json.dumps(toml.load(open(file)))
            elif file.endswith(('.yaml', '.yml')):
                os.environ['YAML_CONFIG'] = json.dumps(yaml.safe_load(open(file)))
            elif file.endswith('.json'):
                os.environ['JSON_CONFIG'] = json.dumps(json.load(open(file)))

            # logger.warning(os.getenv('a'))
            # logger.warning(os.getenv('TOML_CONFIG'))
            # logger.warning(os.getenv('YAML_CONFIG'))
            # logger.warning(os.getenv('JSON_CONFIG'))

        except Exception as e:
            logger.error(e)
            logger.error(traceback.format_exc().strip())


def hot_reload(*paths, is_background: bool = False):
    if is_background:
        background_task(_hot_reload)(*paths)
    else:
        _hot_reload(*paths)


def to_markdown(text):
    from IPython.display import Markdown
    text = text.replace('•', '  *')
    return Markdown(textwrap.indent(text, '> ', predicate=lambda _: True))


def is_jupyter():
    try:
        shell = get_ipython().__class__.__name__
        if shell == 'ZMQInteractiveShell':
            return True  # Jupyter notebook or qtconsole
        elif shell == 'TerminalInteractiveShell':
            return False  # Terminal running IPython
        else:
            return False  # Other type, e.g. python script
    except NameError:
        return False  # Probably standard Python interpreter


def try_tasks(tasks: List[Callable], handle_error: Optional[Callable] = None, *args, **kwargs):
    """
        try_tasks([lambda x: 1 / x, lambda x: 2 / x], handle_error=lambda x: 3 / (x + 0.000001), x=0)
    """
    for i, task in enumerate(tasks):
        try:
            return task(*args, **kwargs)  # 尝试执行操作，如果操作成功，则跳出循环
        except Exception as e:
            _ = f"Task {i} failed: {e}"
            logging.error(_)
            # if is_trace:
            #     logging.error(traceback.format_exc().strip())

    if handle_error:
        return handle_error(*args, **kwargs)


def ttl_fn(ttl: int = 60):
    return time.time() // ttl  # 缓存时间


def storage_to_cookie(storage: Union[str, Path, dict]):
    if isinstance(storage, Path) or (isinstance(storage, str) and len(storage) < 256 and Path(storage).is_file()):
        storage = json.loads(Path(storage).read_bytes())
    elif isinstance(storage, str):
        storage = json.loads(storage)

    if isinstance(storage, dict):
        storage = storage['cookies']

    import jsonpath
    cookies = [f"{n}={v}" for n, v in
               zip(jsonpath.jsonpath(storage, "$..name"), jsonpath.jsonpath(storage, "$..value"))]
    return '; '.join(cookies)


@lru_cache()
def url2fileview(url):
    encoded_url = base64.b64encode(url.encode()).decode()
    # import urllib.parse
    # encoded_url = urllib.parse.quote(encoded_url)

    return f"https://v.ffire.cc/onlinePreview?url={encoded_url}"


def logany(*objs, threshold: int = 1000):
    for obj in objs:
        if len(str(obj)) < threshold:
            logger.debug(obj)


if __name__ == '__main__':
    pass
    # s = "import pandas as pd; output = pd.__version__"
    # s = "import os; output = os.popen(cmd).read().split()"
    # print(exec_callback(s, cmd='ls'))
    # with timer() as t:
    #     time.sleep(3)
    #
    # status, output = magic_cmd('ls')
    # print(status, output)
    #
    # d = {'a': 1, 'b': 2}
    # print(bjson(d))
    # print(BaseConfig.parse_obj(d))

    # print(show_code(show_code))
    # print(get_var_name(s))

    # print(try_tasks([lambda x: 1 / x, lambda x: 2 / x], handle_error=lambda x: 3 / (x + 0.000001), x=0))
    # print(get_function_params(hot_reload))
    # print(url2view("https://sf-maas-uat-prod.oss-cn-shanghai.aliyuncs.com/outputs/dd25c8e0-fc6b-4d9b-aede-f6a489b04dee_00001_.png"))
    # print(get_function_params())

    pass

    # logany("request")

    logany(["request"] * 10)
    logany("reques", "request2")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : playwright_types
# @Time         : 2024/9/18 16:24
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *


class ScreenshotRequest(BaseModel):
    html: str
    width: int = 375
    height: int = 812

    class Config:
        json_schema_extra = {
            "examples": [
                {
                    "html": """
<html>
<head>
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <style>
        body {
            font-family: Arial, sans-serif;
            margin: 0;
            padding: 20px;
        }
        h1 {
            font-size: 24px;
        }
        p {
            font-size: 16px;
        }
        @media (max-width: 600px) {
            h1 {
                font-size: 20px;
            }
            p {
                font-size: 14px;
            }
        }
    </style>
</head>
<body>
    <h1>Hello, World!</h1>
    <p>This is a test with responsive design.</p>
</body>
</html>
                    """
                }
            ]
        }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : __init__.py
# @Time         : 2021/1/20 10:15 上午
# @Author       : yuanjie
# @Email        : meutils@qq.com
# @Software     : PyCharm
# @Description  : 


from meutils.cmds.hdfs_cmd import HDFS







#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : images
# @Time         : 2024/11/13 15:44
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *

import fal_client
from fal_client import AsyncClient, SyncClient

# client = AsyncClient(key=os.getenv("FAL_KEY"))
client = SyncClient(key=os.getenv("FAL_KEY"))

application = "fal-ai/recraft-v3"




handler = client.submit(
    application=application,
    arguments={
        "prompt": "a red panda eating a bamboo in front of a poster that says \"recraft V3 now available at fal\""
    },
)

# 任务id
request_id = handler.request_id
client.get_handle(application, request_id)
client.get_handle(application, request_id).get()



from openai import AsyncClient, Client

resp = Client().images.generate(
    model="dall-e-3",
    prompt="a red panda eating a bamboo in front of a poster that says \"recraft V3 now available at fal\"",
    size="1024x1024",
    quality="standard",
    n=1,
) # {"data": [{"url": '...'}]}


# /replicate/ {

#     openai
#
# }

#
# from
#
# import replicate
#
# output = replicate.run(
#     "black-forest-labs/flux-schnell",
#     input={"prompt": "an iguana on the beach, pointillism"}
# )
#
# # Save the generated image
# with open('output.png', 'wb') as f:
#     f.write(output[0].read())
#
# print(f"Image saved as output.png")





#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : handlers
# @Time         : 2023/8/21 16:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
import notifiers

params = {
    "username": "you@gmail.com",
    "password": "abc123",
    "to": "dest@gmail.com"
}

# Send a single notification
notifier = notifiers.get_notifier("gmail")
notifier.notify(message="The application is running!", **params)

# Be alerted on each error message
from notifiers.logging import NotificationHandler

handler = NotificationHandler("gmail", defaults=params)
logger.add(handler, level="ERROR")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : bar
# @Time         : 2023/3/21 09:20
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 

from meutils.pipe import *
from time import sleep
from gooey import Gooey, GooeyParser
from itertools import cycle


@Gooey(
    terminal_font_family='Courier New',
    progress_regex=r"^progress: (?P<current>-?\d+)/(?P<total>\d+)$",
    progress_expr="current / total * 100",
    timing_options={
        'show_time_remaining': True,
        'hide_time_remaining_on_complete': False
    })
def parse_args():
    parser = GooeyParser(prog="example_progress_bar_5")
    parser.add_argument("alpha", type=float, help="Seconds per 10% progress", default=1)
    parser.add_argument("beta", type=int, help="Extra messages between progress updates", default=3)
    parser.add_argument("gamma", type=str, nargs='+', help="List of your messages",
                        default=["info ...", "message ...", "warning ..."])
    args = parser.parse_args()
    return args.alpha, args.beta, args.gamma


def main():
    alpha, beta, gamma = parse_args()
    myprocess = cycle(gamma)

    print("Step 1")
    for i in range(0, 51, 10):
        sleep(alpha)
        print("progress: {}/{}".format(i, 100), flush=True)

    print("Step 2")
    for i in range(50, 101, 10):
        sleep(alpha / (beta + 1))
        for j in range(beta):
            # some extra text before progress
            print(next(myprocess), flush=True)
            sleep(alpha / (beta + 1))
        print("progress: {}/{}".format(i, 100), flush=True)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : 公网ip
# @Time         : 2022/4/29 上午11:22
# @Author       : yuanjie
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 公网ip


from urllib import request
import re


# 通过sohu公共ip库获取本机公网ip
def get():
    sohu_ip_url = 'http://txt.go.sohu.com/ip/soip'
    r = request.urlopen(sohu_ip_url)
    text = r.read().decode()
    result = re.findall(r'\d+.\d+.\d+.\d+', text)
    if result:
        return result[0]
    else:
        return None


if __name__ == "__main__":
    result = get()
    print(result)


